/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Retrieve the annotated property from the database if the id to retrieve is 
 * included in the current request.
 * To use this, a RetrieveEntityInterceptor must be configured in the Dependency 
 * Injection framework.
 * 
 * @author Ignacio Coloma
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.FIELD, ElementType.METHOD, ElementType.PARAMETER })
@Documented
public @interface RetrieveEntity {

    /** if provided, applies to the property path instead of the annotated property */
    String propertyPath() default "";

    /** the list of events in which this validation takes place, empty for all. Accepts wildcards. */
    String[] on() default {};
    
    /** the list of events in which this validation does NOT place, empty for none. Accepts wildcards. */
    String[] except() default {};
    
    /** 
     * The property to search for. It should identify a result uniquely.
     * If empty, the primary key field will be used. 
     */
    String idPropertyName() default "";
    
    /** 
     * setReadOnly will be invoked right after fetching the object from the database. Objects 
	 * with readOnly = true are still linked to the dataabase, but any changes to the object state 
	 * will not be commited at flush time.
	 * Equivalent to Session.setReadOnly() in Hibernate and UnitOfWork.addReadOnlyClass() in Toplink.
	 * Defaults to false. Do not change unless you know what you are doing.
     */  
    boolean readOnly() default false;
    
}
