package org.loom.annotation.processor;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.List;

import org.loom.annotation.validation.NestedAnnotations;
import org.loom.binding.PropertyWrapper;
import org.loom.binding.PropertyWrapperFactory;
import org.loom.mapping.Event;
import org.loom.mapping.EventParameter;
import org.loom.mapping.VisitedPath;
import org.loom.util.ClassUtils;
import org.loom.util.PropertyUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.AnnotationUtils;

/**
 * Convenience class for annotations that are bound to a property and might be boundmore than once, which 
 * requires some workarounds since this is not possible with java.
 * @author icoloma
 *
 * @param <A> The single annotation class
 * @param <M> The multiple annotation class. It must have a single value property of type A[]
 */
public abstract class AbstractPropertyAnnotationProcessor<A extends Annotation, M extends Annotation> extends AbstractAnnotationProcessor {

	/** the annotation that is processed by this class */
	private Class<A> annotationClass;
	
	/** annotation used as a workaround of the limitation that each java property can only have one annotation of the same class */
	private Class<M> multipleAnnotationClass;

	@Autowired
	private PropertyWrapperFactory propertyWrapperFactory;
	
	protected AbstractPropertyAnnotationProcessor(Class<A> annotationClass, Class<M> multipleAnnotationClass) {
		this.annotationClass = annotationClass;
		this.multipleAnnotationClass = multipleAnnotationClass;
	}
	
	protected AbstractPropertyAnnotationProcessor(Class<A> annotationClass) {
		this(annotationClass, null);
	}

	@Override
	public final void process(Event event) {
		
		// process class attributes
		process(event.getActionMapping().getActionClass(), event, new VisitedPath(null), null, null);
		
		// process event parameters
		for (EventParameter parameter: event.getEventParameters()) {
			if (annotationClass !=  null) {
				A annotation = parameter.getAnnotation(annotationClass);
				if (annotation != null) {
					process(event, annotation, createPropertyPath(parameter.getName(), annotation));
				}
			}
			if (multipleAnnotationClass != null) {
				M multi = parameter.getAnnotation(multipleAnnotationClass);
				if (multi != null) {
					processMultiple(event, multi, parameter.getName());
				}
			}
			NestedAnnotations na = parameter.getAnnotation(NestedAnnotations.class);
			if (na != null) {
				VisitedClasses.add(parameter.getParameterType());
				process(parameter.getParameterType(), event, new VisitedPath(parameter.getName()), na.includeProperties(), na.excludeProperties());
			}
		}
	}
	
	private void process(Class<?> containerClass, Event event, VisitedPath visitedPath, String[] includeProperties, String[] excludeProperties) {
		
		// process class attributes (single annotation)
		if (annotationClass !=  null) {
			List<PropertyWrapper> properties = propertyWrapperFactory.getAnnotatedProperties(containerClass, annotationClass, includeProperties, excludeProperties);
			for (PropertyWrapper property : properties) {
				A annotation = property.getAnnotation(annotationClass);
				if (annotation != null && event.matches(annotation)) {
					process(event, annotation, createPropertyPath(visitedPath, property, annotation));
				}
			}
		}
		
		// process class attributes (multiple annotation)
		if (multipleAnnotationClass !=  null) {
			List<PropertyWrapper> properties = propertyWrapperFactory.getAnnotatedProperties(containerClass, multipleAnnotationClass, includeProperties, excludeProperties);
			for (PropertyWrapper property : properties) {
				M annotation = property.getAnnotation(multipleAnnotationClass);
				if (annotation != null && event.matches(annotation)) {
					processMultiple(event, annotation, createPropertyPath(visitedPath, property, annotation));
				}
			}
		}

		// process nested annotations
		List<PropertyWrapper> properties = propertyWrapperFactory.getAnnotatedProperties(containerClass, NestedAnnotations.class, includeProperties, excludeProperties);
		for (PropertyWrapper property : properties) {
			NestedAnnotations annotation = property.getAnnotation(NestedAnnotations.class);
			if (event.matches(annotation)) {
				Class<?> propertyClass;
				VisitedPath path = new VisitedPath(visitedPath, property.getName());
				if (property.isCollection()) {
					propertyClass = property.getItemMetadata().getItemClass();
				} else if (property.isMap()) {
					propertyClass = property.getItemMetadata(1).getItemClass();
				} else {
					propertyClass = property.getPropertyClass();
				}
				VisitedClasses.add(propertyClass);
				process(propertyClass, event, path, annotation.includeProperties(), annotation.excludeProperties());
					
			}
		}
		
	}

	private <T extends Annotation> String createPropertyPath(VisitedPath visitedPath, PropertyWrapper property, T annotation) {
		return PropertyUtils.concatPropertyPath(visitedPath.getPropertyPath(), property.getName(), (String) AnnotationUtils.getValue(annotation, "propertyPath"));
	}

	/**
	 * Process a single annotation. This method gets invoked once per annotated property or
	 * for eaxch value inside a multiple annotation
	 * @param event the {@link Event} that is being processed
	 * @param annotation the annotation that should be processed
	 * @param propertyPath the property that is annotated with a single or multiple annotation
	 */
	protected abstract void process(Event event, A annotation, String propertyPath);

	/** 
	 * Process a multiple annotation. Internally, invokes process() n times 
	 */
	@SuppressWarnings("unchecked")
	private void processMultiple(Event event, M multiple, String propertyPath) {
		// we must use introspection here
		Method values = ClassUtils.getMethod(multipleAnnotationClass, "value");
		A[] annotations = (A[]) ClassUtils.invoke(multiple, values);
		for (A single : annotations) {
			if (event.matches(single)) {
				process(event, single, createPropertyPath(propertyPath, single));
			}
		}
	}

	/**
	 * Create a  property path concatenating a root property path and an annotation iwth a "propertyPath" attribute
	 */
	private <T extends Annotation> String createPropertyPath(String propertyPath, T annotation) {
		return PropertyUtils.concatPropertyPath(propertyPath, (String) AnnotationUtils.getValue(annotation, "propertyPath"));
	}

	public void setPropertyWrapperFactory(
			PropertyWrapperFactory propertyWrapperFactory) {
		this.propertyWrapperFactory = propertyWrapperFactory;
	}

}