/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding;


import org.loom.converter.Converter;
import org.loom.exception.ConfigException;
import org.loom.i18n.Messages;
import org.loom.i18n.MessagesRepository;
import org.loom.log.Log;
import org.loom.util.PropertyUtils;
import org.loom.validator.PropertyValidator;
import org.loom.validator.ValidationRequest;

/**
 * A binder for a nested property, e.g. address.street.name. 
 * @author icoloma
 */
public class PropertyBinder {

	/** root of the properties in the propertyPath */
	private Node rootNode;
	
	/** end of the chain of nodes in this propertyPath */
	private Node leafNode;
	
	/** the normalized property path as a String */
	private String normalizedPropertyPath;
	
	/** the converter that may be used to set/get text values to this PropertyBinder */
	private Converter converter;
	
	private static Log log = Log.getLog(PropertyBinder.class);
	
	public PropertyBinder(String propertyPath) {
		this.normalizedPropertyPath = propertyPath;
	}

	/**
	 * Assigns a value to the given path
	 * @param rootContainer the object where to assign the value
	 * @param paramValue the value to assign
	 */
	public void setValue(Object rootContainer, String paramName, String paramValue, Messages messages, MessagesRepository repository) {
		assertConverterNotNull();
		
		log.trace("Setting value ", paramValue);
		Object valueAsObject = converter.getAsObject(paramName, paramValue, messages, repository);
		
		// set the value even if there are conversion errors (if that's the case, the populated value will be null). 
		// This way intermediate nodes such as lists and maps will get initialized, even with null values. 
		setValue(rootContainer, paramName, valueAsObject);
	}
	
	public void setValue(Object rootContainer, String propertyPath, Object valueAsObject) {
		rootNode.setValue(rootContainer, valueAsObject, PropertyUtils.split(propertyPath), 0);
	}
	
	/**
	 * Gets the value on the given path
	 * @param rootContainer the root of the property path
	 * @param propertyPath the property path requested 
	 */
	public Object getValue(Object rootContainer, String propertyPath) {
		return rootNode.getValue(rootContainer, PropertyUtils.split(propertyPath), 0);
	}
	
	public String getValueAsText(Object rootContainer, String propertyPath, MessagesRepository repository) {
		assertConverterNotNull();
		Object value = rootNode.getValue(rootContainer, PropertyUtils.split(propertyPath), 0);
		return converter.getAsText(value, repository);
	}
	
	private void assertConverterNotNull() {
		if (converter == null) {
			throw new ConfigException("Cannot set/get value of " + normalizedPropertyPath + " because its converter is null. Please add any missing generic type, or alternatively consider using @CollectionMetadata, @Converter or @MapMetadata");
		}
	}
	
	public Node getRootNode() {
		return rootNode;
	}

	public void setRootNode(Node rootNode) {
		this.rootNode = rootNode;
		this.leafNode = rootNode;
		while (!leafNode.isLeaf()) {
			leafNode = leafNode.getChild();
		}
	}

	public String getNormalizedPropertyPath() {
		return normalizedPropertyPath;
	}

	/**
	 * Applies a validation. If there is more than one value involved in this validation (i.e. lists,
	 * maps, or arrays), validates each and every node.
	 * @param container the action whose properties will be validated.
	 * @param validator The validator to be applied
	 * @param messages where to put any validation messages that may occur
	 */
	public void applyValidation(Object container, PropertyValidator validator, Messages messages) {
		applyValidation(container, new ValidationRequest(validator, messages));
	}
	
	public void applyValidation(Object container, ValidationRequest request) {
		rootNode.applyValidation(container, request);
	}

	/**
	 * 
	 * @return the property class of the end of this propertyPath
	 */
	public Class<?> getPropertyClass() {
		return leafNode.getPropertyWrapper().getPropertyClass();
	}
	
	public Node getLeafNode() {
		return leafNode;
	}

	public Converter getConverter() {
		return converter;
	}

	public void setConverter(Converter converter) {
		this.converter = converter;
	}
	
}
