/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding;

import java.lang.annotation.Annotation;
import java.lang.reflect.AnnotatedElement;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.TypeVariable;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.loom.binding.factories.ImplementationFactory;
import org.loom.util.ClassUtils;
import org.springframework.core.GenericCollectionTypeResolver;
import org.springframework.core.MethodParameter;


/**
 * Wraps a property, allowing to read and write it in an encapsulated way
 * 
 * @author icoloma
 *
 */
public class PropertyWrapper implements AnnotationContainer {

	/** the container class (the class that has the property) */
	private Class<?> containerClass;
	
	/** the property name */
	private String propertyName;
	
	/** the property class */
	private Class<?> propertyClass;

	/** the property value factory */
	private ImplementationFactory<?> implementationFactory;
	
	/** the list of annotations of this property. It's the sum of the getter, setter and field annotations */
	private Annotation[] annotations = new Annotation[] {};
	
	/** the getter method */
	private Method getter;
	
	/** the setter method */
	private Method setter;
	
	/** the field used for direct access, if there is none of the above */
	private Field field;
	
	/** the value to be used when assigning null values (0 in the case of int fields, for example */
	private Object nullValue;
	
	/** 
	 * the list of item metadata specified by SetMetadata, CollectionMetadata or MapMetadata. It is handled in the same order as generics parameters.
	 * Lists have a single value that corresponds to the list item type, and Maps have two values that are the key type and the value type, respectively. 
	 */
	private ItemMetadata[] itemMetadata; 

	public PropertyWrapper(Class<?> containerClass, String propertyName) {
		this.containerClass = containerClass;
		this.propertyName = propertyName;
	}
	
	/**
	 * @return the given Annotation if it's assigned to this property, null if none
	 */
	@SuppressWarnings("unchecked")
	public <T extends Annotation> T getAnnotation(Class<T> annotationClass) {
		for (Annotation annotation : annotations) {
			if (annotationClass.isAssignableFrom(annotation.getClass())) {
				return (T) annotation;
			}
		}
		return null;
	}
	
	/**
	 * Adds the element annotations to this property wrapper
	 * @param element
	 */
	public void addAnnotations(AnnotatedElement element) {
		if (element != null) {
			Annotation[] elementAnnotations = element.getAnnotations();
			int length = elementAnnotations.length;
			if (length > 0) {
				Annotation[] newAnnotations = new Annotation[annotations.length + length];
				System.arraycopy(annotations, 0, newAnnotations, 0, annotations.length);
				System.arraycopy(elementAnnotations, 0, newAnnotations, annotations.length, length);
				this.annotations = newAnnotations;
			}
		}
	}
	
	/**
	 * Gets the property value
	 * @param container The object that contains the property
	 * @param createIfNull if true, creates a new instance if the property value is null
	 * @return The property value
	 */
	public Object get(Object container, boolean createIfNull) {
		Object value = get(container);
		if (value == null && createIfNull) {
			if (isReadOnly()) {
				throw new IllegalArgumentException("Cannot assign value to " + getDisplayName() + " because it is read-only. Either an attribute of setter method is required.");
			}
			if (implementationFactory == null) {
				throw new IllegalArgumentException("No implementation factory found for " + 
						getDisplayName() + ", cannot instantiate attribute of type: " + getDefaultPropertyClass().getName() +
						". Please use @ImplementationMetadata or a concrete class (not abstract or interface)");
			}

			value = implementationFactory.newInstance();
			set(container, value);
		}
		return value;
	}
	
	public void setField(Field field) {
		this.field = field;
		if (field != null) {
			addAnnotations(field);
			if (getter == null) {
				if (!Modifier.isPublic(field.getModifiers())) {
					field.setAccessible(true);
				}
				setPropertyClass(field.getType(), field.getGenericType());
			}
		}
	}

	public void setGetter(Method getter) {
		this.getter = getter;
		if (getter != null) {
			setPropertyClass(getter.getReturnType(), getter.getGenericReturnType());
			addAnnotations(getter);
		}
	}

	public void setSetter(Method setter) {
		this.setter = setter;
		if (setter != null) {
			if (propertyClass == null) {
				setPropertyClass(setter.getParameterTypes()[0]);
			}
			addAnnotations(setter);
		}
	}
	
	/**
	 * Assign the propertyClass value. If the provided generic type uses generic variables, tries to resolve them.
	 * Example: 
	 * <pre>
	 * public class AbstractClass<T extends Serializable> {
	 *    private T field;
	 * }
	 * public class MyClass extends AbstractClass<Integer> {
	 * }
	 * </pre>
	 * In this case, "field" would provide ( pc=Serializable.class, genericType=T ), which this  method should resolve
	 * to use Integer instead of Serializable  for this property field. 
	 * 
	 * @param pc the property class value
	 * @param genericType the generic declaration of this class, if applies
	 */
	@SuppressWarnings("unchecked")
	private void setPropertyClass(Class<?> pc, Type genericType) {
		propertyClass = pc;
		
		// if this property type is parameterized
		if (genericType instanceof TypeVariable && containerClass.getGenericSuperclass() instanceof ParameterizedType) { 
			TypeVariable<?>[] types = containerClass.getSuperclass().getTypeParameters();
			Type[] actualTypes = ((ParameterizedType)containerClass.getGenericSuperclass()).getActualTypeArguments();
			TypeVariable<?> propertyType = (TypeVariable<?>) genericType; 
			
			for (int i = 0; i < types.length; i++) {
				if (propertyType.getName().equals(types[i].getName())) {
					if (actualTypes[i] instanceof Class) {
						propertyClass = (Class<?>) actualTypes[i];
					}
					break;
				}
			}
		}
	}

	
	/**
	 * @return the value to this property in the specified container
	 */
	protected Object get(Object container) {
		if (getter != null) {
			return ClassUtils.invoke(container, getter);
		}
		if (field != null) {
			return ClassUtils.getFieldValue(container, field);
		}
		throw new IllegalArgumentException("Cannot return value for property " + containerClass.getSimpleName() + 
				"." + propertyName + ". There is no attribute/getter method defined.");
	}

	/**
	 * Sets a value to this property in the specified container
	 */
	public void set(Object container, Object value) {
		if (value == null && nullValue != null) {
			value = nullValue;
		}
		if (setter != null) {
			ClassUtils.invoke(container, setter, value);
		} else if (field != null) {
			ClassUtils.setFieldValue(container, field, value);
		} else {
			throw new IllegalArgumentException("Cannot set value for property " + containerClass.getSimpleName() + 
					"." + propertyName + ". There is no attribute/setter method defined.");
		}
	}
	
	public boolean isCollection() {
		return Collection.class.isAssignableFrom(propertyClass) || propertyClass.isArray(); 
	}
	
	public boolean isMap() {
		return Map.class.isAssignableFrom(propertyClass); 
	}
	
	public boolean isSet() {
		return Set.class.isAssignableFrom(propertyClass); 
	}

	private Class<?> getDefaultPropertyClass() {
		return getter != null? getter.getReturnType() : field.getType();
	}
	
	public Field getField() {
		return field;
	}
	
	public Method getGetter() {
		return getter;
	}
	
	public Method getSetter() {
		return setter;
	}
	
	public boolean isReadOnly() {
		return field == null && setter == null;
	}
	
	/**
	 * @return the array of annotations associated to the property
	 */
	public Annotation[] getAnnotations() {
		return annotations;
	}
	
	public Class<?> getContainerClass() {
		return containerClass;
	}

	public Class<?> getPropertyClass() {
		return propertyClass;
	}

	public String getPropertyName() {
		return propertyName;
	}
	
	public String getName() {
		return getPropertyName();
	}

	public void setContainerClass(Class<?> containerClass) {
		this.containerClass = containerClass;
	}

	public void setPropertyClass(Class<?> propertyClass) {
		this.propertyClass = propertyClass;
	}
	
	/**
	 * @return the name used to display error or informational messages
	 */
	public String getDisplayName() {
		return containerClass.getSimpleName() + "." + propertyName;
	}

	public void setPropertyName(String propertyName) {
		this.propertyName = propertyName;
	}

	public ImplementationFactory<?> getImplementationFactory() {
		return implementationFactory;
	}

	public void setImplementationFactory(ImplementationFactory<?> valueFactory) {
		this.implementationFactory = valueFactory;
	}
	
	/** for debugging purposes */
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append(getClass().getSimpleName());
		builder.append('@').append(Integer.toHexString(hashCode()));
		builder.append(" { containerClass = ").append(containerClass.getSimpleName());
		builder.append(", propertyName = ").append(propertyName);
		builder.append(", propertyClass = ").append(propertyClass.getSimpleName());
		builder.append(", annotations = [").append(StringUtils.join(annotations, ", ")).append("]");
		if (getter != null) {
			builder.append(", getter = ").append(getter.getName());
		}
		if (setter != null) {
			builder.append(", setter = ").append(setter.getName());
		}
		if (field != null) {
			builder.append(", field = ").append(field);
		}
		builder.append("}");
		return builder.toString();
	}

	public Object getNullValue() {
		return nullValue;
	}

	public void setNullValue(Object nullValue) {
		this.nullValue = nullValue;
	}

	public void setItemMetadata(ItemMetadata itemMetadata) {
		this.itemMetadata = new ItemMetadata[] { itemMetadata };
	}
	
	public void setItemMetadata(ItemMetadata[] itemMetadata) {
		this.itemMetadata = itemMetadata;
	}

	public ItemMetadata getItemMetadata() {
		return getItemMetadata(0);
	}
	
	public ItemMetadata getItemMetadata(int index) {
		return itemMetadata == null?  null : itemMetadata[index];
	}
	
	public int getItemMetadataSize() {
		return itemMetadata == null?  0 : itemMetadata.length;
	}

	/**
	 * @return the container class for the next node in a property chain. For simple nodes this would be the same as the property class, but
	 * collections will return the item class, and Maps the value class.
	 */
	public Class<?> getNextPropertyClass() {
		return itemMetadata == null? propertyClass : itemMetadata[itemMetadata.length - 1].getItemClass();
	}

	/**
	 * Guess the collection generic type of this property, if possible. Return null if it cannot be guessed.
	 */
	public Class<?> guessCollectionGenericType() {
		if (!Collection.class.isAssignableFrom(propertyClass)) {
			throw new IllegalStateException("The property is not a Collection");
		}
		if (getter != null) {
			return GenericCollectionTypeResolver.getCollectionReturnType(getter);
		} else if (setter != null) {
			return GenericCollectionTypeResolver.getCollectionParameterType(new MethodParameter(setter, 0));
		} else if (field != null) {
			return GenericCollectionTypeResolver.getCollectionFieldType(field);
		}
		return null;
	}

	/**
	 * Guess the Map generic type used for the key of this Map property, String.class if it cannot be guessed.
	 */
	public Class<?> guessMapGenericKeyType() {
		if (!Map.class.isAssignableFrom(propertyClass)) {
			throw new IllegalStateException("The property is not a Map");
		}
		if (getter != null) {
			return GenericCollectionTypeResolver.getMapKeyReturnType(getter);
		} else if (setter != null) {
			return GenericCollectionTypeResolver.getMapKeyParameterType(new MethodParameter(setter, 0));
		} else if (field != null) {
			return GenericCollectionTypeResolver.getMapKeyFieldType(field);
		}
		return String.class;
	}

	/**
	 * Guess the Map generic type used for the value of this Map property, String.class if it cannot be guessed.
	 */
	public Class<?> guessMapGenericValueType() {
		if (!Map.class.isAssignableFrom(propertyClass)) {
			throw new IllegalStateException("The property is not a Map");
		}
		if (getter != null) {
			return GenericCollectionTypeResolver.getMapValueReturnType(getter);
		} else if (setter != null) {
			return GenericCollectionTypeResolver.getMapValueParameterType(new MethodParameter(setter, 0));
		} else if (field != null) {
			return GenericCollectionTypeResolver.getMapValueFieldType(field);
		}
		return String.class;
	}

}
