/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.config;

import static org.loom.config.ClassNames.JAVAEE_5;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletContext;

import org.codehaus.jackson.map.ObjectMapper;
import org.loom.annotation.SSLPolicyType;
import org.loom.binding.PropertyWrapperFactory;
import org.loom.exception.DefaultExceptionHandler;
import org.loom.exception.ExceptionHandler;
import org.loom.i18n.MessagesRepositoryFactory;
import org.loom.log.Log;
import org.loom.mapping.ActionMappingRepository;
import org.loom.resolution.DefaultResolutionFactoryImpl;
import org.loom.resolution.ResolutionFactory;
import org.loom.resources.WebResourceBundleFactory;
import org.loom.resources.WebResourceBundleFactoryImpl;
import org.loom.resources.WebResourceBundleRepository;
import org.loom.servlet.LoomServletRequestFactory;
import org.loom.servlet.params.DefaultFileParameterFactory;
import org.loom.servlet.params.FileParameterFactory;
import org.loom.tags.ExpressionLanguageProxy;
import org.loom.tags.Jee5ExpressionLanguageProxy;
import org.loom.tags.LegacyExpressionLanguageProxy;
import org.loom.tags.decorator.TagDecoratorRepository;
import org.loom.util.ClassUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.context.support.ServletContextResourcePatternResolver;

/**
 * Loom configuration. The configuration class must be unique per servlet, but
 * could be more than one per web container
 * 
 * @author icoloma
 */
public class Config {

	/** repository with all configured ActionMapping instances */
	private ActionMappingRepository actionMappingRepository;

	@Autowired(required=false)
	private LoomServletRequestFactory loomServletRequestFactory;

	/** creates uploaded file items (adapter for commons-fileupload) */
	@Autowired(required=false)
	private FileParameterFactory fileParameterFactory;

	/** TagDecorator instances repository */
	@Autowired(required=false)
	private TagDecoratorRepository tagDecoratorRepository;

	/** all the cleanup tasks that should be executed on undeploy */
	private static List<Runnable> undeployTasks = new ArrayList<Runnable>();

	/** the repository of {@link WebResource} to use */
	@Autowired(required=false)
	private WebResourceBundleRepository webResourceBundleRepository;

	/** the class that executes EL expressions inside loom */
	@Autowired(required=false)
	private ExpressionLanguageProxy elProxy;
	
	@Autowired(required=false)
	private PropertyWrapperFactory propertyWrapperFactory;
	
	/** true to use development defaults, defaults to true */
	private boolean development = true;
	
	/** the {@link ResolutionFactory} to be used for creating Redirect and Forward resolutions */
	@Autowired(required=false)
	private ResolutionFactory resolutionFactory;
	
	/** the default SSL policy, defaults to {@link SSLPolicyType}.DO_NOT_MODIFY */
	private SSLPolicyType defaultSSLPolicy = SSLPolicyType.DO_NOT_MODIFY;
	
	/** the {@link ObjectMapper} used to serialize/deserialize JSON objects */
	@Autowired(required=false)
	private ObjectMapper objectMapper;

	@Autowired(required=false)
	private ExceptionHandler exceptionHandler;
	
	private ServletContext servletContext;
	
	private static Log log = Log.getLog(Config.class);

	/** the singleton instance */
	private static Config instance;

	/**
	 * Validates that this config object has been properly configured
	 */
	public void validate() {

		if (propertyWrapperFactory == null) {
			throw new IllegalArgumentException("A configured PropertyWrapperFactory instance is required");
		}
		if (fileParameterFactory == null) {
			fileParameterFactory = new DefaultFileParameterFactory();
		}
		if (objectMapper == null) {
			objectMapper = new ObjectMapper()
				.configure(org.codehaus.jackson.JsonParser.Feature.ALLOW_UNQUOTED_FIELD_NAMES, true)
				.configure(org.codehaus.jackson.JsonParser.Feature.ALLOW_SINGLE_QUOTES, true);
		}
		if (loomServletRequestFactory == null) {
			loomServletRequestFactory = new LoomServletRequestFactory();
		}
		loomServletRequestFactory.setServletContext(servletContext);
		loomServletRequestFactory.validate();
		if (tagDecoratorRepository == null) {
			tagDecoratorRepository = new TagDecoratorRepository();
		}
		if (elProxy == null) {
			if (ClassUtils.isPresent(JAVAEE_5)) {
				elProxy = new Jee5ExpressionLanguageProxy(); 
				((Jee5ExpressionLanguageProxy)elProxy).setServletContext(servletContext);
			} else {
				elProxy = new LegacyExpressionLanguageProxy();
			}
		}
		if (resolutionFactory == null) {
			resolutionFactory = new DefaultResolutionFactoryImpl();
			((DefaultResolutionFactoryImpl)resolutionFactory).setMapper(objectMapper);
		}
		if (webResourceBundleRepository == null) {
			WebResourceBundleFactoryImpl factory = new WebResourceBundleFactoryImpl();
			ServletContextResourcePatternResolver resourceLoader = new ServletContextResourcePatternResolver(servletContext);
			factory.setResourceLoader(resourceLoader);
			factory.initDefaultCompilers();
			setWebResourceBundleFactory(factory);
		}
		if (exceptionHandler == null) {
			exceptionHandler = new DefaultExceptionHandler();
		}
		
		// initialize development mode on all dependent configuration classes
		setDevelopment(development);

	}
	

	/**
	 * Scans for actions and configure the corresponding ActionMappings
	 */
	public void scanAndConfigureActions() {
		actionMappingRepository.scanAndRegisterActions();
	}

	/**
	 * Adds a cleanup task to be executed when the war gets undeployed
	 */
	public static void addUndeployTask(Runnable task) {
		if (task instanceof Thread) {
			throw new IllegalArgumentException(
					"Allocating Threads that are not going to be launched using start() "
							+ "can be a source of memory leaks. Be nice an implement Runnable instead.");
		}
		undeployTasks.add(task);
	}

	/**
	 * Executes the undeploy tasks
	 */
	public static void cleanup() {
		for (Runnable task : undeployTasks) {
			try {
				task.run();
			} catch (Exception e) {
				log.error(e, e);
			}
		}
	}

	public ActionMappingRepository getActionMappingRepository() {
		return actionMappingRepository;
	}

	public static final Config getInstance() {
		return instance;
	}

	public static final void setInstance(Config config) {
		Config.instance = config;
	}

	public TagDecoratorRepository getTagDecoratorRepository() {
		return tagDecoratorRepository;
	}

	public void setActionMappingRepository(ActionMappingRepository actionMappingRepository) {
		this.actionMappingRepository = actionMappingRepository;
	}

	public void setTagDecoratorRepository(TagDecoratorRepository tagDecoratorRepository) {
		this.tagDecoratorRepository = tagDecoratorRepository;
	}

	public boolean isDevelopment() {
		return development;
	}

	public synchronized void setDevelopment(boolean development) {
		this.development = development;
		if (webResourceBundleRepository != null) {
			this.webResourceBundleRepository.setDevelopment(development);
		}
	}

	public ExpressionLanguageProxy getElProxy() {
		return elProxy;
	}

	public ResolutionFactory getResolutionFactory() {
		return resolutionFactory;
	}

	public void setResolutionFactory(ResolutionFactory resolutionFactory) {
		this.resolutionFactory = resolutionFactory;
	}

	public SSLPolicyType getDefaultSSLPolicy() {
		return defaultSSLPolicy;
	}

	public void setDefaultSSLPolicy(SSLPolicyType defaultSSLPolicy) {
		this.defaultSSLPolicy = defaultSSLPolicy;
	}

	public ObjectMapper getObjectMapper() {
		return objectMapper;
	}

	public void setObjectMapper(ObjectMapper objectMapper) {
		this.objectMapper = objectMapper;
	}

	public FileParameterFactory getFileParameterFactory() {
		return fileParameterFactory;
	}

	public void setFileParameterFactory(FileParameterFactory fileParameterFactory) {
		this.fileParameterFactory = fileParameterFactory;
	}

	public PropertyWrapperFactory getPropertyWrapperFactory() {
		return propertyWrapperFactory;
	}
	
	public void setPropertyWrapperFactory(PropertyWrapperFactory propertyWrapperFactory) {
		this.propertyWrapperFactory = propertyWrapperFactory;
	}

	public ExceptionHandler getExceptionHandler() {
		return exceptionHandler;
	}

	public void setExceptionHandler(ExceptionHandler exceptionTranslator) {
		this.exceptionHandler = exceptionTranslator;
	}

	public void setElProxy(ExpressionLanguageProxy elProxy) {
		this.elProxy = elProxy;
	}
	
	@Autowired(required=false)
	public void setWebResourceBundleFactory(WebResourceBundleFactory factory) {
		if (webResourceBundleRepository == null) {
			webResourceBundleRepository = new WebResourceBundleRepository();
		}
		webResourceBundleRepository.setWebResourceBundleFactory(factory);
	}

	public WebResourceBundleRepository getWebResourceBundleRepository() {
		return webResourceBundleRepository;
	}

	public void setWebResourceBundleRepository(
			WebResourceBundleRepository webResourceBundleRepository) {
		this.webResourceBundleRepository = webResourceBundleRepository;
	}


	public LoomServletRequestFactory getLoomServletRequestFactory() {
		return loomServletRequestFactory;
	}


	public void setLoomServletRequestFactory(
			LoomServletRequestFactory loomServletRequestFactory) {
		this.loomServletRequestFactory = loomServletRequestFactory;
	}


	public ServletContext getServletContext() {
		return servletContext;
	}


	public void setServletContext(ServletContext servletContext) {
		this.servletContext = servletContext;
	}

	/**
	 * @deprecated Use getLoomServletRequestFactory().getMessagesRepositoryFactory() instead
	 */
	public MessagesRepositoryFactory getMessagesRepositoryFactory() {
		return loomServletRequestFactory.getMessagesRepositoryFactory();
	}

	/**
	 * @deprecated Use getLoomServletRequestFactory().getLocaleResolver() instead
	 */
	public LocaleResolver getLocaleResolver() {
		return loomServletRequestFactory.getLocaleResolver();
	}

}
