package org.loom.config;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.springframework.core.io.Resource;

/**
 * Observes a list of Resources and trigger an event if one of them gets modified.
 * This method will observe only Resources that are accessible in the filesystem (i.e., 
 * not inside jars or WARs)
 * 
 * @author icoloma
 */
public class ResourcesObserver {

	/** the list of observed Resources */
	private List<ResourceTimestamp> resources;

	/** the listener to notify of modification events */
	private ResourcesModificationListener listener;

	/**
	 * @param listener the resources listener whose files should be observed. Note that 
	 * only files that exist in the filesystem (not inside a WAR file) will be observed.
	 */
	public ResourcesObserver(ResourcesModificationListener listener) {
		this.listener = listener;
		this.resources = new ArrayList<ResourceTimestamp>();
		for (Resource resource : listener.getResources()) {
			try {
				this.resources.add(new ResourceTimestamp(resource));
			} catch (IOException e) {
				// Ignore. There is no other way to guess if it's a real file... 
			}
		}
	}
	
	/**
	 * Checkk all the observed Resources for  modifications. If any of them has been modified, triggers 
	 * the modification event.
	 */
	public void checkResources() {
		List<Resource> modifiedResources = null;
		for (ResourceTimestamp timestamp : resources) {
			if (timestamp.checkForUpdates()) {
				if (modifiedResources == null) {
					modifiedResources = new ArrayList<Resource>();
				}
				modifiedResources.add(timestamp.getResource());
			}
		}
		if (modifiedResources != null) {
			listener.resourcesModified(modifiedResources);
		}
	}
	
	public ResourcesModificationListener getListener() {
		return listener;
	}
	
	private class ResourceTimestamp {
		
		/** the Resource that is being observed */
		private Resource resource; 

		/** the observed File */
		private File file;
		
		/** the last checked timestamp */
		private long lastTimestamp;
		
		public ResourceTimestamp(Resource resource) throws IOException {
			this.resource = resource;
			this.file = resource.getFile();
			checkForUpdates();
		}
		
		/**
		 * Check if the Resource has changed since the last check() invocation
		 * @return true if the file has been updated, false otherwise
		 */
		public boolean checkForUpdates() {
			long last = lastTimestamp;
			lastTimestamp = file.lastModified();
			return last != lastTimestamp;
		}

		public Resource getResource() {
			return resource;
		}

	}
	
}
