/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.interceptor;

import java.util.Set;

import javax.imageio.ImageIO;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.i18n.Message;
import org.loom.i18n.Messages;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.params.FileParameter;
import org.loom.util.HtmlSanitizer;
import org.loom.util.StringUtils;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;


/**
 * Validate one image parameter
 * This class and its descendants are thread-safe.
 * 
 * @author Rafael Serrano
 */
public abstract class AbstractImageValidationInterceptor extends FileValidationInterceptor {

	/** minimum width */
    private Integer minWidth;

    /** maximum width */
    private Integer maxWidth;

    /** minimum height */
    private Integer minHeight;
    
    /** maximum height */
    private Integer maxHeight;
    
    @Override
	protected void validateParameterExtensionPoint(LoomServletRequest request, String currentParameterName, FileParameter fileParam) {
		
		Messages messages = request.getParsedAction().getMessages();
		ImageData imageData = getImageData(fileParam); 
		boolean valid = imageData != null && (allowedFormats.isEmpty() || allowedFormats.contains(imageData.getFormat()));
		Message message = null;
		if (valid) {
			
			if (minWidth != null && imageData.getWidth() < minWidth || minHeight != null && imageData.getHeight() < minHeight) {
				message = messages.error(currentParameterName, "loom.validation.imageMinFailed")
					.addArg("minWidth", minWidth == null? 1 : minWidth)
					.addArg("minHeight", minHeight == null? 1 : minHeight);
			} else if (maxWidth != null && imageData.getWidth() > maxWidth || maxHeight != null && imageData.getHeight() > maxHeight) {
				message = messages.error(currentParameterName, "loom.validation.imageMaxFailed");
	    		if (maxHeight == null) {
					message.addTranslatedArg("maxHeight", "loom.validation.unlimited");
				} else {
					message.addArg("maxHeight", maxHeight);
				}
	    		if (maxWidth == null) {
					message.addTranslatedArg("maxWidth", "loom.validation.unlimited");
				} else {
					message.addArg("maxWidth", maxWidth);
				}
			}
			if (message != null) {
				message.addArg("width", imageData.getWidth());
				message.addArg("height", imageData.getHeight());
			}
		} else {
			Set<String> formats = allowedFormats.isEmpty()? Sets.newTreeSet(Lists.newArrayList(ImageIO.getReaderFormatNames())) : allowedFormats;
			message = messages.error(currentParameterName, "loom.validation.imageFormatFailed")
				.addArg("currentFormat", imageData == null || StringUtils.isEmpty(imageData.getFormat())? "" : " (" + imageData.getFormat() + ")")
				.addArg("validFormats", StringUtils.join(formats, ", "));
		}
		if (message != null) {
			message.addArg("maxFileSize", maxFileSize)
				.addArg("filename", HtmlSanitizer.sanitize(fileParam.getFilename()));
		}
	}

    /**
     * @return the image data of the uploaded file, null if the file is not valid
     */
    protected abstract ImageData getImageData(FileParameter fileParam);


    @Override
    public String toString() {
    	return new ToStringBuilder(this)
    		.append("parameterName", parameterName)
    		.append("minWidth", minWidth)
    		.append("maxWidth", maxWidth)
    		.append("minHeight", minHeight)
    		.append("maxHeight", maxHeight)
    		.append("allowedFormats", allowedFormats)
    		.toString();
    }

	@Override
	public void setAllowedFormats(String[] allowedFormats) {
		for (String format : allowedFormats) {
			this.allowedFormats.add(format.toLowerCase());
		}
	}

	public Integer getMinWidth() {
		return minWidth;
	}

	public void setMinWidth(Integer minWidth) {
		this.minWidth = minWidth;
	}

	public Integer getMaxWidth() {
		return maxWidth;
	}

	public void setMaxWidth(Integer maxWidth) {
		this.maxWidth = maxWidth;
	}

	public Integer getMinHeight() {
		return minHeight;
	}

	public void setMinHeight(Integer minHeight) {
		this.minHeight = minHeight;
	}

	public Integer getMaxHeight() {
		return maxHeight;
	}

	public void setMaxHeight(Integer maxHeight) {
		this.maxHeight = maxHeight;
	}

	@Override
	public Integer getMaxFileSize() {
		return maxFileSize;
	}

	@Override
	public void setMaxFileSize(Integer maxFileSize) {
		this.maxFileSize = maxFileSize;
	}

	@Override
	public void setAllowedFormats(Set<String> allowedFormats) {
		this.allowedFormats = allowedFormats;
	}

	@Override
	public void setParameterName(String parameterName) {
		this.parameterName = parameterName;
	}

	@Override
	public Set<String> getAllowedFormats() {
		return allowedFormats;
	}

	@Override
	public String getParameterName() {
		return parameterName;
	}
	
}
