package org.loom.interceptor;

import java.lang.reflect.Method;

import javax.servlet.http.HttpServletResponse;

import org.loom.log.Log;
import org.loom.mapping.ParsedAction;
import org.loom.resolution.CacheableResolution;
import org.loom.resolution.HttpErrorResolution;
import org.loom.resolution.Resolution;
import org.loom.servlet.CacheControl;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.names.RequestAttributeNames;
import org.loom.util.ClassUtils;

/**
 * Intercepts requests and invoke the associated method to get the Cache information.
 * If the browser cached data is ok, returns a 304 and interupts the workflow, else
 * let the event execute normally.
 * 
 * @author icoloma
 */
public class CacheInterceptor implements ExecuteInterceptor, ResolutionInterceptor {

	/** the method that returns the CacheControl instance after invocation */
	private Method cacheMethod;
	
	private static Log log = Log.getLog(CacheInterceptor.class);

	public Resolution beforeExecute(ParsedAction action) {
		CacheControl cacheControl = calculateCacheControl(action);
		LoomServletRequest request = action.getAction().getContext().getRequest();
		if (!cacheControl.isResourceModified(request)) {
			HttpErrorResolution resolution = new HttpErrorResolution(HttpServletResponse.SC_NOT_MODIFIED);
			resolution.setCacheControl(cacheControl);
			log.trace("Browser-supplied headers are still valid. Returning 304");
			return resolution;
		} 
		log.debug("Browser-supplied headers are out-of-date. Proceeding to the event method");
		request.setAttribute(RequestAttributeNames.CACHE_CONTROL, cacheControl);
		return null;
	}

	/**
	 * Execute the method to retrieve the CacheControl instance.
	 * @param action the current action
	 * @return the CacheControl instance.
	 */
	protected CacheControl calculateCacheControl(ParsedAction action) {
		return (CacheControl) ClassUtils.invoke(action.getAction(), cacheMethod);
	}
	
	public void beforeResolution(ParsedAction action, Resolution resolution) {
		// if there is no resolution or the list of error messages is not empty,
		// the cache will not apply
		if (resolution == null || !action.getMessages().isEmpty()) {
			return;
		}
		if (!(resolution instanceof CacheableResolution)) {
			throw new RuntimeException("The resolution instance must implement CacheableResolution, but it does not: " + resolution.getClass().getName());
		}
		LoomServletRequest request = action.getAction().getContext().getRequest();
		CacheControl cacheControl = (CacheControl) request.getAttribute(RequestAttributeNames.CACHE_CONTROL);
		((CacheableResolution)resolution).setCacheControl(cacheControl);
	}
	
	public Method getCacheMethod() {
		return cacheMethod;
	}

	public void setCacheMethod(Method cacheMethod) {
		this.cacheMethod = cacheMethod;
	}

}
