/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.json;

import java.lang.reflect.Array;
import java.util.BitSet;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;
import org.codehaus.jackson.map.ObjectMapper;
import org.joda.time.DateTime;

/**
 * Generates a JSON representation of the underlying HashMap, in the most 
 * readable form possible.
 * That means, if the object has a single property, uses an assignment. Else, 
 * use object extension (Object.extend(...)) or JSON object declaration.
 * @deprecated use jackson {@link ObjectMapper} instead
 * @author icoloma
 */
public class JsonMarshaller {

	private Map<String, Object> contents = new LinkedHashMap<String, Object>(); 
	
	/** true to prepend "var ", false otherwise */
	private boolean declare = true;
	
	/** name of the javascript object */
	private String id;
	
	/** true to use quotes for every property name, default false. If false, it will only use quotes for properties that are not valid javascript identifiers */
	private boolean quotePropertyNames = false;
	
	/** the list of characters are valid JSON identifiers */
	private static BitSet VALID_PROPERTYNAME_CHARS;
	
	static {
		VALID_PROPERTYNAME_CHARS = new BitSet();
		for (char ch = 'a'; ch <= 'z'; ch++) {
			VALID_PROPERTYNAME_CHARS.set(ch);
			VALID_PROPERTYNAME_CHARS.set(Character.toUpperCase(ch));
		}
		for (char ch = '0'; ch <= '9'; ch++) {
			VALID_PROPERTYNAME_CHARS.set(ch);
		}
	}
	
	public JsonMarshaller() {
	}
	
	public JsonMarshaller(String name) {
		this.id = name;
	}
	
	public JsonMarshaller(String name, Map<String, ?> contents) {
		this.id = name;
		for (Map.Entry<String, ?> entry : contents.entrySet()) {
			add(entry.getKey(), entry.getValue());
		}
	}

	/**
	 * @return the JSON representation of this object
	 */
	@Override
	public String toString() {
		if (contents.size() == 0) {
			return "{}";
		}
		StringBuilder builder = new StringBuilder(contents.size() * 20);
		if (id != null) {
			if (declare) {
				builder.append("var ");
			}
			builder.append(id).append(" = ");
		}
		buildBody(builder);
		builder.append("\n");
		return builder.toString();
	}

	/**
	 * 
	 * @param builder
	 */
	public void buildBody(StringBuilder builder) {
		builder.append("{\n");
		for (Iterator<Map.Entry<String, Object>> i = contents.entrySet().iterator(); i.hasNext();) {
			Map.Entry<String, Object> entry = i.next();
			String key = entry.getKey();
			appendPropertyName(builder, key);
			builder.append(": ");
			appendPropertyValue(builder, entry.getValue());
			if (i.hasNext()) {
				builder.append(",");
			}
			builder.append("\n");
		}
		builder.append("}");
	}
	
	/**
	 * Serialize a property value and appends it to the StringBuilder passed as argument
	 */
	@SuppressWarnings("unchecked")
	public static void appendPropertyValue(StringBuilder builder, Object object) {
		if (object instanceof String) {
			String s = (String) object;
			builder.append('"').append(StringUtils.replace(s, "\"", "\\\"")).append('"');
		} else if (object instanceof Character) {
				builder.append('\"').append(object).append('\"');
		} else if (object instanceof DateTime) {
			appendDateValue(builder, ((DateTime)object).getMillis());
		} else if (object instanceof Date) {
			appendDateValue(builder, ((Date)object).getTime());
		} else if (object instanceof Pattern) {
			Pattern pattern = (Pattern) object;
			builder.append('/').append(pattern.pattern()).append('/');
			boolean caseSensitive = (pattern.flags() & Pattern.CASE_INSENSITIVE) == 0;
			if (caseSensitive) {
				builder.append('i');
			}
		} else if (object instanceof JsonMarshaller) {
			((JsonMarshaller)object).buildBody(builder);
		} else if (object instanceof Map) {
			JsonMarshaller nested = new JsonMarshaller();
			nested.addAll((Map<String, ?>)object);
			nested.buildBody(builder);
		} else if (object instanceof Iterable) {
			builder.append('[');
			for (Iterator i = ((Iterable)object).iterator(); i.hasNext();) {
				appendPropertyValue(builder, i.next());
				if (i.hasNext()) {
					builder.append(',');
				}
			}
			builder.append(']');
		} else if (object.getClass().isArray()) {
			builder.append('[');
			int length = Array.getLength(object);
			for (int i = 0; i < length; i++) {
				appendPropertyValue(builder, Array.get(object, i));
				if (i < length - 1 ) {
					builder.append(',');
				}
			}
			builder.append(']');
		} else {
			builder.append(object.toString());
		}
		
	}
	
	private static void appendDateValue(StringBuilder builder, long millis) {
		builder.append("new Date(\"").append(millis).append("\")");
	}

	private void appendPropertyName(StringBuilder builder, String key) {
		if (!quotePropertyNames && isValidIdentifier(key)) {
			builder.append(key);
		} else {
			builder.append("\"").append(key).append("\"");
		}
	}
	
	/**
	 * @return true if the key is a valid property identifier, false if not 
	 * (if false, the property should be enclosed with quotes)
	 */
	private boolean isValidIdentifier(String key) {
		int length = key.length();
        for (int i = 0; i < length; i++) {
            char ch = key.charAt(i);
            if (!VALID_PROPERTYNAME_CHARS.get(ch)) {
            	return false;
            }
        }
		return true;
	}

	public void setId(String name) {
		this.id = name;
	}

	/**
	 * @return true if there is any property in this Json object
	 */
	public boolean isEmpty() {
		return contents.isEmpty();
	}

	public String getId() {
		return id;
	}
	
	public Object get(String propertyName) {
		return contents.get(propertyName);
	}

	/**
	 * Add all entries in a Map
	 */
	public JsonMarshaller addAll(Map<String, ?> entries) {
		contents.putAll(entries);
		return this;
	}
	
	/**
	 * Add one attribute to this JSON object
	 * @param name name of the attribute
	 * @param value value
	 */
	public JsonMarshaller add(String name, Object value) {
		if (value != null) {
			contents.put(name, value);
		}
		return this;
	}
	
	/**
	 * Append an expression such as <code>max: MyClass.calculate()</code>
	 */
	public JsonMarshaller addExpression(String name, String expression) {
		if (expression != null) {
			contents.put(name, new Expression(expression));
		}
		return this;
	}
	
	private class Expression {
		
		private String expression;

		public Expression(String expression) {
			this.expression = expression;
		}
		
		@Override
		public String toString() {
			return expression;
		}
	}

	public void setDeclare(boolean declare) {
		this.declare = declare;
	}

	public boolean isQuotePropertyNames() {
		return quotePropertyNames;
	}

	public void setQuotePropertyNames(boolean quoteAllPropertyNames) {
		this.quotePropertyNames = quoteAllPropertyNames;
	}
	
}
