package org.loom.mapping;

import java.text.ParsePosition;
import java.util.Map;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * A token in the uri path.
 * @author icoloma
 *
 */
public class UriToken {

	/** true if this token is required to be a valid URI, default true */
	private boolean required = true;
	
	/** the String that is recognized as the start of this token */
	private String handle;
	
	/** the name of this param value, null if there is no parameter attached (just the handle) */
	private String name;
	
	/** The actions that are indicated to the parser */
	public enum ParserAction {

		/** proceed to the next token */
		NEXT_TOKEN, 
		
		/** fail: the input does not match the expected format */
		FAIL, 
		
		/** finish processing the input */
		END
		
	}
	
	@Override
	public String toString() {
		return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
			.append("handle", handle)
			.append("name", name)
			.append("required", required).toString();
			
	}
	
	/**
	 * Extract the next ParamValue from the uri that is being parsed
	 * @param uri the uri that is being parsed
	 * @param pp the current position. If there is a match, its current index will be incremented
	 * pointing to the first character after this token.
	 * @param params the list of extracted parameters. If this token is a match, the parsed value (if any) will be added.
	 * @param nextToken the next token, null if none.
	 * @return true if the token was a match, false if not.
	 */
	public ParserAction match(String uri, ParsePosition pp, Map<String, String> params, UriToken nextToken) {
		
		int uriIndex = pp.getIndex();
		int handleIndex = 0;
		
		// if we only have optional parameters and the uri is already finished, this is a success
		if (uriIndex == uri.length() && !isRequired()) {
			return ParserAction.END;
		}
		
		// parse the handle
		while (handleIndex < handle.length()) {
			
			// if the uri is too short or simply does not contain the handle, fail
			if (uriIndex >= uri.length() || uri.charAt(uriIndex) != handle.charAt(handleIndex)) {
				return ParserAction.FAIL;
			}
			uriIndex++;
			handleIndex++;
		}
		
		// if the handle is OK, parse the param value
		int endIndex = -1;
		if (name != null) {
			if (nextToken !=  null) {
				endIndex = uri.indexOf(nextToken.getHandle(), uriIndex);
			} 
			if (endIndex == -1) {
				endIndex = uri.length();
			}
			if (uriIndex < endIndex) { // add the value only if not empty
				String value = uri.substring(uriIndex, endIndex);
				params.put(name, value);
			} else if (required) {
				return ParserAction.FAIL;
			}
		} else {
			endIndex = uriIndex;
		}
		
		// increment ParsePosition and return parsed ParamValue
		pp.setIndex(endIndex);
		return ParserAction.NEXT_TOKEN;
	}

	public boolean isRequired() {
		return required;
	}

	public void setRequired(boolean required) {
		this.required = required;
	}

	public String getHandle() {
		return handle;
	}

	public void setHandle(String handle) {
		this.handle = handle;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
}
