package org.loom.persistence;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;

import org.loom.exception.EntityNotFoundException;
import org.loom.paged.PagedListCriteria;
import org.loom.paged.PagedListData;

/**
 * Interface that combines EntityManager operations and extended operations.
 * @author icoloma
 *
 */
public interface ExtendedEntityManager extends EntityManager {

	/**
     * Retrieve a paged query. The query can include a sorting order, which would then
     * be appended to the sort indicated by the page object, if any.
	 * @param <T> The type of the result items
	 * @param criteria The criteria of the paged query: page index, page size, sorting, etc.
	 * @return The paged result, sorted and paged according to the provided arguments.
	 */
	public <T> PagedListData<T> query(PagedListCriteria criteria);
 

	/**
	 * Retrieves a non-paged query.
	 * Use with care, this method could potentially instantiate large amounts of data.
	 * @param <T> The type of the result items
	 * @param queryString the query to execute
	 * @param params the parameters to be added to the query, in order 
	 * @return The result of executing the query
	 */
	public <T> List<T> find(String queryString, Object... params);
	
	/**
	 * @param queryString the query to execute
	 * @param params the parameters to be added to the query, in order 
	 * @return a single object that satisfies the query.
     * @throws NoResultException if there is no result
     * @throws NonUniqueResultException if there is more than one result
     * @throws IllegalStateException if called for a Java 
     *    Persistence query language UPDATE or DELETE statement
	 */
	public <T> T findSingle(String queryString, Object... params);

	/**
	 * Removes a persistent instance
	 * @param <T> The persistent class
	 * @param clazz The persistent class
	 * @param id the primary key to remove
	 * @return the removed instance
	 * @throws EntityNotFoundException if the entity could not be found
	 */
	public <T> T remove(Class<T> clazz, Object id);
	
	/**
	 * Retrieves all records for a specified persistent class
	 * This method should be used with care, as it could instantiate huge amounts of data. 
	 * @param <T> The persistent class to retrieve
	 * @param clazz The persistent class to retrieve
	 * @return The list of all records found on the database
	 */
	public <T> List<T> findAll(Class<T> clazz);

	/**
	 * Retrieves a single persistent instance from the database
	 * @param <T> The class to retrieve
	 * @param clazz The class to retrieve
	 * @param id The primary key to retrieve
	 * @return The retrieved object
	 * @throws EntityNotFoundException if the enity could not be retrieved
	 */
	public <T> T find(Class<T> clazz, Object id);
	
	/**
	 * Retrieves a non-paged query
	 * This method should be used with care, as it could instantiate huge amounts of data. 
	 * @param <T> The type of the result items
	 * @param namedQuery the named query to execute
	 * @param params the parameters to be added to the query, in order 
	 * @return The result of executing the query
	 */
	public <T> List<T> findNamed(String namedQuery, Object... params);
	
	/**
	 * Execute a query that retrieves a single object.
	 * This method should be used with care, as it could instantiate huge amounts of data. 
	 * @param <T> The type of the result items
	 * @param namedQuery the named query to execute
	 * @param params the parameters to be added to the query, in order 
	 * @return a single object that satisfies the named query.
     * @throws NoResultException if there is no result
     * @throws NonUniqueResultException if more than one result
     * @throws IllegalStateException if called for a Java 
     *    Persistence query language UPDATE or DELETE statement
	 */
	public <T> T findNamedSingle(String namedQuery, Object... params);
	
	/**
	 * Execute a query that updates the database.
	 * @param queryString the update/delete/insert query to execute
	 * @param params the parameters to be added to the query, in order 
	 * @return the number of records that were modified
	 */
	public int update(String queryString, Object... params);

	/** 
	 * @param clazz the persistent class
	 * @return the name of the id property of a persistent class 
	 * @throws UnsupportedOperationException if this feature has not been implemented
	 */
	public String getIdPropertyName(Class<?> clazz);

	/**
	 * @param clazz the persistent class
	 * @return the version property name of a persistent class if it has defined one, null otherwise
	 * @throws UnsupportedOperationException if this feature has not been implemented
	 */
	public String getVersionPropertyName(Class<?> clazz);
	
	/**
	 * @param entity the persistent entity to inspect
	 * @return the version value of a persistent instance, or null if not versioned
	 * @throws UnsupportedOperationException if this feature has not been implemented
	 */
	public <T> T getVersionValue(Object entity);

	/**
	 * Remove this instance from the session cache. Changes to the instance will
	 * not be synchronized with the database.
	 * @param entity a persistent instance
	 */
	public void evict(Object entity);

	/**
	 * Sets an instance to readOnly. ReadOnly instances do not write any changes
	 * to the database when flushing.
	 * @param entity a persistent instance
	 * @param readOnly true to set the entity to readOnly
	 */
	public void setReadOnly(Object entity, boolean readOnly);
	
}
