package org.loom.persistence;

import java.util.List;

import javax.persistence.EntityTransaction;
import javax.persistence.FlushModeType;
import javax.persistence.LockModeType;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.loom.paged.PagedListCriteria;
import org.loom.paged.PagedListData;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * An ExtendedEntityManager wrapper that offer transactional access to address common needs.
 * @author icoloma
 *
 */
@Service
public class TransactionalService implements ExtendedEntityManager {

	/** the ExtendedEntityManager instance */
	@PersistenceContext
	private ExtendedEntityManager entityManager;

	/* ------+------+------ transactional operations ------+------+------ */
	
	@Transactional
	public void flush() {
		entityManager.flush();
	}
	
	@Transactional
	public void lock(Object entity, LockModeType lockMode) {
		entityManager.lock(entity, lockMode);
	}

	@Transactional
	public <T> T merge(T entity) {
		return entityManager.merge(entity);
	}

	@Transactional
	public void persist(Object entity) {
		entityManager.persist(entity);
	}

	@Transactional
	public <T> T remove(Class<T> clazz, Object id) {
		return entityManager.remove(clazz, id);
	}

	@Transactional
	public void remove(Object entity) {
		entityManager.remove(entity);
	}

	@Transactional
	public int update(String queryString, Object... params) {
		return entityManager.update(queryString, params);
	}

	/* ------+------+------ non-transactional operations ------+------+------ */

	public void clear() {
		entityManager.clear();
	}

	public boolean contains(Object entity) {
		return entityManager.contains(entity);
	}

	public void evict(Object entity) {
		entityManager.evict(entity);
	}

	public <T> T find(Class<T> entityClass, Object primaryKey) {
		return entityManager.find(entityClass, primaryKey);
	}

	public <T> List<T> find(String queryString, Object... params) {
		return entityManager.find(queryString, params);
	}

	public <T> List<T> findAll(Class<T> clazz) {
		return entityManager.findAll(clazz);
	}

	public <T> List<T> findNamed(String namedQuery, Object... params) {
		return entityManager.findNamed(namedQuery, params);
	}

	public <T> T findNamedSingle(String namedQuery, Object... params) {
		return entityManager.<T>findNamedSingle(namedQuery, params);
	}

	public <T> T findSingle(String queryString, Object... params) {
		return entityManager.<T>findSingle(queryString, params);
	}
	
	public FlushModeType getFlushMode() {
		return entityManager.getFlushMode();
	}

	public String getIdPropertyName(Class<?> clazz) {
		return entityManager.getIdPropertyName(clazz);
	}

	public <T> T getReference(Class<T> entityClass, Object primaryKey) {
		return entityManager.getReference(entityClass, primaryKey);
	}
	
	public Object getDelegate() {
		return entityManager;
	}
	
	public String getVersionPropertyName(Class<?> clazz) {
		return entityManager.getVersionPropertyName(clazz);
	}

	public <T> T getVersionValue(Object entity) {
		return entityManager.<T>getVersionValue(entity);
	}
	
	public <T> PagedListData<T> query(PagedListCriteria criteria) {
		return entityManager.query(criteria);
	}
	
	public void refresh(Object entity) {
		entityManager.refresh(entity);
	}
	
	public void setReadOnly(Object entity, boolean readOnly) {
		entityManager.setReadOnly(entity, readOnly);
	}
	
	public void setEntityManager(ExtendedEntityManager entityManager) {
		this.entityManager = entityManager;
	}
	
	/* ------+------+------ unsupported operations ------+------+------ */
	
	public Query createNamedQuery(String name) {
		throw new UnsupportedOperationException();
	}

	@SuppressWarnings("unchecked")
	public Query createNativeQuery(String sqlString, Class resultClass) {
		throw new UnsupportedOperationException();
	}

	public Query createNativeQuery(String sqlString, String resultSetMapping) {
		throw new UnsupportedOperationException();
	}

	public Query createNativeQuery(String sqlString) {
		throw new UnsupportedOperationException();
	}

	public Query createQuery(String qlString) {
		throw new UnsupportedOperationException();
	}
	
	public EntityTransaction getTransaction() {
		throw new UnsupportedOperationException();
	}

	public void close() {
		throw new UnsupportedOperationException();
	}
	
	public void setFlushMode(FlushModeType flushMode) {
		throw new UnsupportedOperationException();
	}
	
	public boolean isOpen() {
		throw new UnsupportedOperationException();
	}
	
	public void joinTransaction() {
		throw new UnsupportedOperationException();
	}

}
