/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.persistence.file;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.DigestOutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import javax.persistence.PersistenceContext;

import org.apache.commons.io.IOUtils;
import org.loom.exception.ConfigException;
import org.loom.exception.EntityNotFoundException;
import org.loom.log.Log;
import org.loom.persistence.ExtendedEntityManager;
import org.loom.servlet.params.FileParameter;
import org.loom.util.StringUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.transaction.annotation.Transactional;

/**
 * Stores file data in a database table, and file contents in a configured system folder. 
 * @author Ignacio Coloma
 */
public class ExternalFileManager implements FileManager, InitializingBean {
	
	/** the JPA entity manager */
	@PersistenceContext
	private ExtendedEntityManager entityManager;

	/** the folder to store uploaded files, system temp dir by default */
	private File rootFolder;

	/** maximum number of files per folder, default to Integer.MAX_VALUE */
	private int maxFilesPerFolder = Integer.MAX_VALUE;
	
	private static Log log = Log.getLog(ExternalFileManager.class);
	
	public void afterPropertiesSet() throws Exception {
		if (rootFolder == null) {
			throw new ConfigException("storageRoot is required");
		}
	}
	
	@Transactional
	public PersistentFile merge(FileParameter parameter) {
		return merge(new PersistentFile(parameter));
	}
	
	@Transactional
	public PersistentFile merge(PersistentFile file) {
		OutputStream output = null;
		InputStream input = null;
		try {
			input = file.getContentsAsStream();
			file = entityManager.merge(file);
			
			// save contents
			File f = getFileLocation(file);
			log.debug("Saving file to ", f.getPath());
			MessageDigest digest = MessageDigest.getInstance("MD5");
			output = new DigestOutputStream(new FileOutputStream(f), digest);
			IOUtils.copy(input, output);
			
			file.setContents(new SystemFileContents(f));
			file.setMD5(StringUtils.toString(digest));
			return file;
			
		} catch (IOException e) {
			throw new RuntimeException(e);
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(output);
			IOUtils.closeQuietly(input);
		}
	}

	/**
	 * Return the file location of a persistent file.
	 * this implementation just returns the file ID, relative to the repository root.
	 */
	protected File getFileLocation(PersistentFile file) {
		File containingFolder = getContainingFolder(file.getId());
		return new File(containingFolder, file.getId().toString());
	}
	
	public File getContainingFolder(int id) {
		int folder = id / maxFilesPerFolder;
		File location = new File(rootFolder, String.valueOf(folder));
		if (!location.exists()) {
			if (!location.mkdirs()) {
			throw new RuntimeException("Could not create the Alexandria repository folder '" + location.getAbsolutePath() + 
			"'. Please check that all required permissions are set, or override the project root by adding -Dalexandria.home=<new root> to the server startup script");
			}
			log.debug("Created folder: ", location.getAbsolutePath());
		} 
		return location;
	}
	
	public PersistentFile find(Integer id) {
		log.debug("Retrieving file with key ", id);
		PersistentFile file = entityManager.find(PersistentFile.class, id); 
		if (file == null) {
			throw new EntityNotFoundException(PersistentFile.class, id);
		}
		File location = getFileLocation(file);
		if (!location.exists()) {
			throw new EntityNotFoundException(PersistentFile.class, id, "File " + id + " exists in the database, but external location " + location.getAbsolutePath() + " is not found");
		}
		file.setContents(new SystemFileContents(location));
		return file;
	}

	@Transactional
	public PersistentFile remove(Integer id) {
		log.debug("Removing file with key ", id);
		PersistentFile file = entityManager.remove(PersistentFile.class, id);
		File f = getFileLocation(file);
		if (!f.delete()) {
			log.warn("Could not delete ", f.getPath(), ". It will be done on JVM exit");
			f.deleteOnExit();
		}
		return file;
	}

	/**
	 * @deprecated use setRootFolderLocation() instead
	 */
	public void setStorageRoot(String storageRoot) {
		setRootFolderLocation(storageRoot);
	}
	
	public void setRootFolderLocation(String location) {
		if (location == null) {
			rootFolder = null;
		} else {
			rootFolder = new File(location);
			if (!rootFolder.exists() && !rootFolder.mkdirs()) {
				throw new RuntimeException("Could not create the repository root folder '" + location + "'. Please check that all required permissions are set");
			}
		}

	}
	
	public void setEntityManager(ExtendedEntityManager entityManager) {
		this.entityManager = entityManager;
	}

	public File getRootFolder() {
		return rootFolder;
	}

	public void setRootFolder(File rootFolder) {
		this.rootFolder = rootFolder;
	}

	public int getMaxFilesPerFolder() {
		return maxFilesPerFolder;
	}

	public void setMaxFilesPerFolder(int maxFilesPerFolder) {
		this.maxFilesPerFolder = maxFilesPerFolder;
	}

}
