/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resolution;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.Cookie;

import org.loom.exception.HttpException;
import org.loom.log.Log;
import org.loom.servlet.CacheControl;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;


/**
 * Forward a request. Forwarded resources are located by default inside /WEB-INF/jsp to
 * prevent direct browser access.
 * 
 * @author Ignacio Coloma
 */
public class ForwardResolutionImpl extends AbstractHttpResolution implements ForwardResolution {

	/** the url to forward to */
	private String destination;

	/** the root of forwarding pages. Empty to use the web root */
	private String forwardResourcesRoot;
	
	/** the list of attributes set */
	private Map<String, Object> attributes;
	
	private static Log log = Log.getLog(ForwardResolutionImpl.class);
	
	@Override
	public void resolve(LoomServletRequest request, LoomServletResponse response) throws IOException, ServletException {
		if (attributes != null) {
			for (Map.Entry<String, Object> entry : attributes.entrySet()) {
				request.setAttribute(entry.getKey(), entry.getValue());
			}
		}
		
		String target = calculateTargetResource(request);
		log.debug("Forwarding request to ", target);
		RequestDispatcher requestDispatcher = request.getRequestDispatcher(target);
		if (requestDispatcher == null) {
			throw new HttpException(404, "Cannot forward to " + target + " (file not found)");
		}
		writeHeaders(request, response);
		requestDispatcher.forward(request, response);
	}
	
	public String calculateTargetResource(LoomServletRequest request) {
		if (destination.length() > 0 && destination.charAt(0) == '/') {
			return forwardResourcesRoot + destination;
		} else {
			return forwardResourcesRoot + request.getParsedAction().getActionMapping().getForwardFolder() + "/" + destination;
		}
	}

	public String getDestination() {
		return destination;
	}

	public void setDestination(String resource) {
		this.destination = resource;
	}
	
	@Override
	public String toString() {
		return "ForwardResolution(" + destination + ")";
	}

	public String getForwardResourcesRoot() {
		return forwardResourcesRoot;
	}

	public void setForwardResourcesRoot(String forwardResourcesRoot) {
		this.forwardResourcesRoot = forwardResourcesRoot;
	}

	public ForwardResolution set(String name, Object value) {
		if (attributes == null) {
			attributes = new HashMap<String, Object>();
		}
		attributes.put(name, value);
		return this;
	}
	
	public ForwardResolution setAttribute(String name, Object value) {
		return set(name, value);
	}
	
	@Override
	public ForwardResolution addCookie(String name, String value) {
		return (ForwardResolution) super.addCookie(name, value);
	}
	
	@Override
	public ForwardResolution addCookie(String name, String value, int maxAge) {
		return (ForwardResolution) super.addCookie(name, value, maxAge);
	}
	
	@Override
	public ForwardResolution addCookie(Cookie cookie) {
		return (ForwardResolution) super.addCookie(cookie);
	}
	
	@Override
	public ForwardResolution addHeader(String name, Object value) {
		return (ForwardResolution) super.addHeader(name, value);
	}
	
	@Override
	public ForwardResolution setHeader(String name, Object value) {
		return (ForwardResolution) super.setHeader(name, value);
	}
	
	@Override
	public ForwardResolution setHeaderIfNotPresent(String name, Object value) {
		return (ForwardResolution) super.setHeaderIfNotPresent(name, value);
	}
	
	@Override
	public ForwardResolution setCharset(String charset) {
		return (ForwardResolution) super.setCharset(charset);
	}
	
	@Override
	public ForwardResolution setContentType(String contentType) {
		return (ForwardResolution) super.setContentType(contentType);
	}
	
	@Override
	public ForwardResolution setCacheControl(CacheControl cacheControl) {
		return (ForwardResolution) super.setCacheControl(cacheControl);
	}
}
