package org.loom.resources;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.SortedMap;

import javax.annotation.PostConstruct;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;

import com.google.common.collect.Maps;


public class PrecompiledWebResourceBundleFactoryImpl implements WebResourceBundleFactory {

	/** the compiler to use for js files */
	private WebResourceBundleFactory devFactory;
	
	@Autowired
	private ResourceLoader resourceLoader;
	
	/** true if we are in development mode, false otherwise */
	private boolean development;
	
	/** the compiled index contents (key is the bundlename, value is the resource location) */
	private SortedMap<String, String> bundles = Maps.newTreeMap();
	
	/** the compiled index properties file location */
	private Resource compiledIndex;
	
	/** the url where to write the compiled resources (relative to the webapp root), default "compiled" */
	private String destinationFolder = "compiled/";

	@PostConstruct
	public void initIndex() {
		if (this.devFactory == null) {
			this.devFactory = initDefaultFactory();
		}
		if (compiledIndex == null) {
			compiledIndex = resourceLoader.getResource(destinationFolder + "resources-index.properties");
		}
		if (!compiledIndex.exists()) {
			throw new IllegalArgumentException(compiledIndex.getDescription() + " does not exist. You must precompile your resources first.");
		}
		InputStream input = null;
		try {
			input = compiledIndex.getInputStream();
			Properties props = new Properties();
			props.load(input);
			for (Map.Entry<Object, Object> entry : props.entrySet()) {
				String name = (String) entry.getKey();
				String location = (String) entry.getValue();
				bundles.put(name, location);
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(input);
		}
	}
	
	protected WebResourceBundleFactory initDefaultFactory() {
		WebResourceBundleFactoryImpl factory = new WebResourceBundleFactoryImpl();
		factory.setResourceLoader(resourceLoader);
		factory.initDefaultCompilers();
		factory.setDevelopment(true);
		return factory;
	}

	@Override
	public WebResourceBundle create(String n) {
		String name = StringUtils.substringAfterLast(n, "/");
		if (development) {
			WebResourceBundle result = devFactory.create(n);
			return result;
		} else {
			String location = bundles.get(name);
			if (location == null) {
				throw new IllegalArgumentException("No bundle found with name=" + name + ". Maybe you should recompile your web resources");
			}
			return new PrecompiledWebResourceBundle(name, location);
		}
	}

	@Override
	public WebResourceBundle create(String name, List<Resource> resources) {
		throw new UnsupportedOperationException("Precompiled resources can only be defined using using @include from jsp pages or tag files");
	}

	@Override
	public void setDevelopment(boolean development) {
		this.development = development;
	}

	public void setDestinationFolder(String destinationFolder) {
		this.destinationFolder = destinationFolder.endsWith("/")? destinationFolder : destinationFolder + "/";
	}

	public void setResourceLoader(ResourceLoader resourceLoader) {
		this.resourceLoader = resourceLoader;
	}

}
