package org.loom.resources;

import java.io.IOException;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.jsp.JspException;

import org.loom.action.support.ResourcesAction;
import org.loom.exception.HttpException;
import org.loom.tags.core.AbstractWebResourceTag;
import org.springframework.core.io.Resource;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

public class WebResourceBundleImpl implements WebResourceBundle {
	
	/** the name of this instance */
	private String name;
	
	/** the list of source resource locations */
	private Set<Resource> resources = new LinkedHashSet<Resource>();

	/** the type of this bundle */
	private WebResourceType type;
	
	/** the compiler to use */
	private WebResourceCompiler compiler;

	/** the list of compiled web resources to serve. compiledWebResources.length != resources.length */
	private Map<String, CompiledWebResource> compiledWebResources;
	
	/** 
	 * if type == CSS, this is the list of images referenced by url() attributes in the CSS files. 
	 * The key value is the name of the image url
	 * For JS bundles its value is null 
	 */
	private Map<String, CssResource> cssExternalResources;
	
	/** Listeners to the ResourceBundleCompiledEvent */
	private List<WebResourceBundleEventListener> listeners = Lists.newArrayList();
	
	@Override
	public void render(AbstractWebResourceTag tag) throws JspException, IOException {
		if (compiledWebResources == null) {
			initWebResources();
		}
		tag.getUrl().replace(ResourcesAction.BUNDLE, name);
		for (CompiledWebResource res : compiledWebResources.values()) {
			res.render(tag);
		}
	}
	
	/**
	 * this.compiledWebResources is not transient on purpose, since double instantiation 
	 * is not really a problem.
	 * We're just trying to be clean here, but without hurting performance for the common case.
	 */
	private synchronized void initWebResources() {
		if (compiledWebResources == null) {
			CompilationResult compileResult = compiler.compile(name, resources);
			compiledWebResources = Maps.newLinkedHashMap();
			for (CompiledWebResource res : compileResult.getCompiledResources()) {
				compiledWebResources.put(res.getName(), res);
			}
			if (compileResult.getCssExternalResources() != null) {
				cssExternalResources = Maps.newHashMapWithExpectedSize(compileResult.getCssExternalResources().size());
				for (CssResource cssResource : compileResult.getCssExternalResources()) {
					cssExternalResources.put(cssResource.getName(), cssResource);
				}
			}
			for (WebResourceBundleEventListener listener : listeners) {
				listener.onCompile(this, compileResult);
			}
		}
	}
	
	public CompiledWebResource getCompiledWebResource(String name) {
		if (compiledWebResources == null) {
			initWebResources();
		}
		CompiledWebResource result = compiledWebResources.get(name);
		if (result == null) {
			throw new HttpException(404, "CompiledWebResource with name '" + name + "' not found in bundle '" + name + "'");
		}
		return result;
	}
	
	public CssResource getCssResource(String name) {
		CssResource result = cssExternalResources.get(name);
		if (result == null) {
			throw new HttpException(404, "CssExternalResource with name '" + name + "' not found in bundle '" + name + "'");
		}
		return result;
	}
	
	@Override
	public void reload() {
		this.compiledWebResources = null;
		this.cssExternalResources = null;
	}

	public void setResources(Set<Resource> resources) {
		this.resources = resources;
		type = WebResourceType.identify(resources.iterator().next());
	}
	
	@Override
	public void addWebResourceBundleEventListener( WebResourceBundleEventListener listener) {
		listeners.add(listener);
	}

	public WebResourceType getType() {
		return type;
	}

	public void setType(WebResourceType type) {
		this.type = type;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void setCompiler(WebResourceCompiler compiler) {
		this.compiler = compiler;
	}

	public String getName() {
		return name;
	}
	
}
