package org.loom.resources;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.springframework.core.io.Resource;

import com.google.common.collect.Lists;

/**
 * Default implementation of WebResourceBundle 
 * @author icoloma
 */
public class WebResourceBundleRepository {
	
	/** the list of configured WebResourceBundle, the name is key */
	private Map<String, WebResourceBundle> bundles = new ConcurrentHashMap<String, WebResourceBundle>();
	
	private WebResourceBundleFactory webResourceBundleFactory;
	
	/** Listeners to the ResourceBundleCompiledEvent */
	private List<WebResourceBundleEventListener> listeners = Lists.newArrayList();

	/**
	 * @param name the name of the WebResourceBundle
	 * @throws IllegalArgumentException if no bundle  could be found with the given name
	 * @return the WebResourceBundle with the given name
	 */
	public WebResourceBundle getWebResourceBundle(String name) {
		WebResourceBundle bundle = bundles.get(name);
		if (bundle == null) {
			bundle = webResourceBundleFactory.create(name);
			for (WebResourceBundleEventListener listener : listeners) {
				bundle.addWebResourceBundleEventListener(listener);
			}
			
			// store under the actual location and under the bundle name
			bundles.put(name, bundle);
			bundles.put(bundle.getName(), bundle);
		}
		return bundle;
	}
	
	/**
	 * Add a WebResourceBundle
	 * @param name the name of the resource bundle
	 * @param resources the list of resources that integrate the bundle.
	 */
	public WebResourceBundle addResourceBundle(String name, List<Resource> resources) {
		WebResourceBundle bundle = webResourceBundleFactory.create(name, resources);
		bundles.put(name, bundle);
		return bundle;
	}
	
	public void setBundles(Collection<WebResourceBundle> bundles) {
		for (WebResourceBundle bundle : bundles) {
			this.bundles.put(bundle.getName(), bundle);
		}
	}

	public void setWebResourceBundleFactory( WebResourceBundleFactory webResourceBundleFactory) {
		this.webResourceBundleFactory = webResourceBundleFactory;
	}

	public void setDevelopment(boolean development) {
		webResourceBundleFactory.setDevelopment(development);
		for (WebResourceBundle bundle : bundles.values()) {
			bundle.reload();
		}
	}
	
	public void addWebResourceBundleEventListener(WebResourceBundleEventListener listener) {
		listeners.add(listener);
		for (WebResourceBundle bundle : bundles.values()) {
			bundle.addWebResourceBundleEventListener(listener);
		}
	}

}
