/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.resources;

import org.loom.servlet.names.ContentTypeNames;
import org.springframework.core.io.Resource;

/**
 * Type of web resource.
 * If used for a concatenated list of resources, it contains the type of the first identified resource
 * @author icoloma
 */
public enum WebResourceType {
	
	/** javascript file @see {@link ContentTypeNames}*/
	JS("application/javascript"),
	
	/** css file @see {@link ContentTypeNames} */
	CSS("text/css"), 
	
	/** unknown file extension */
	UNKNOWN(null);
	
	/** the content type of this resource type */
	private String contentType;
	
	/** the extension used for this type of files */
	private String extension;
	
	WebResourceType(String contentType) {
		this.contentType = contentType;
		this.extension = '.' + this.toString().toLowerCase();
		if (".unknown".equals(this.extension)) {
			this.extension = "";
		}
	}
	
	/**
	 * @return the WebResourceType for the specified location, according to its extension.
	 * If the extension is missing or unrecognized, returns UNKNOWN.
	 */
	public static WebResourceType identify(Resource resource) {
		return identify(resource.getFilename());
	}
	
	public static final WebResourceType identify(String location) {
		if (location.endsWith(".js")) {
			return JS;
		} else if (location.endsWith(".css")) {
			return CSS;
		} else {
			return UNKNOWN;
		}
	}
	

	public String getContentType() {
		return contentType;
	}

	public String getExtension() {
		return extension;
	}
	
}
