/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.servlet;

import static org.loom.servlet.names.RequestAttributeNames.FLASH_NEXT;
import static org.loom.servlet.names.RequestAttributeNames.FLASH_PREV;
import static org.loom.servlet.names.RequestParameterNames.METHOD_OVERRIDES;

import java.util.Enumeration;
import java.util.Locale;

import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.loom.action.Action;
import org.loom.i18n.MessagesRepository;
import org.loom.log.Log;
import org.loom.mapping.ParsedAction;
import org.loom.servlet.names.RequestAttributeNames;
import org.loom.servlet.names.RequestHeaderNames;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.servlet.params.FileParameter;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.servlet.scope.FlashContext;
import org.loom.servlet.scope.FlashContextFactory;
import org.loom.tags.core.FormTag;
import org.loom.url.UrlBuilder;
import org.loom.util.CookieUtils;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.Iterators;
import com.google.common.collect.ListMultimap;


/**
 * The real request is always wrapped inside one of these. This objects wraps
 * all the attributes required by Loom at any time, with special attention to:
 * <ul>
 * <li>the source URL: this attributes keeps track of the URL that originated
 * this request
 * 
 * @author icoloma
 */
public class LoomServletRequestImpl extends HttpServletRequestWrapper implements LoomServletRequest {

	/** the current MessagesRepository */
	private MessagesRepository messagesRepository;

	/** the current parsed action */
	private ParsedAction parsedAction;

	/** the list of GET/POST params */
	private ServletRequestParameters params;

	/** the currently processed form tag, null if none */
	private FormTag currentForm;
	
	/** the current HTML id to use for id generation */
	private int generatedIdCount = 0;
	
	/** the user overriden locale (using the Locale cookie), if any */
	private Locale locale;
	
	/** the recognized browser features */
	private BrowserData browserData;
	
	/** the request HTTP method */
	private HttpMethod method;
	
	private ServletContext servletContext;
	
	private FlashContextFactory flashContextFactory;

	/** scopes that can be accessed from a request object */
	static Scope[] VALID_SCOPES;
	
	/** bind the current request to the current thread */
	private static ThreadLocal<LoomServletRequest> threadLocal = new ThreadLocal<LoomServletRequest>();
	
	private static Log log = Log.getLog(LoomServletRequestImpl.class);
	
	static {
		Scope[] values = Scope.values();
		VALID_SCOPES = new Scope[values.length - 1];
		System.arraycopy(values, 1, VALID_SCOPES, 0, VALID_SCOPES.length);
	}

	LoomServletRequestImpl(ServletRequest req, LoomServletRequestData data) {
		super((HttpServletRequest) req);
		this.servletContext = data.getServletContext();
		this.flashContextFactory = data.getFlashContextFactory();
		this.setMessagesRepository(data.getMessagesRepository());
		
		HttpServletRequest nested = (HttpServletRequest) req;
		this.params = new ServletRequestParameters(data.getDefaultCharset(), nested);
		this.params.restoreFromRedirect(this); // restore old parameters from a redirect
		
		this.browserData = new BrowserData(nested.getHeader(RequestHeaderNames.USER_AGENT));
		
		// initialize the HTTP method. Override only if this is a POST request, to protect against GET forgery
		String m = nested.getParameter(METHOD_OVERRIDES);
		this.method = HttpMethod.valueOf(m != null && "POST".equals(nested.getMethod())? m.toUpperCase() : nested.getMethod());
		
		// log the contents of this request
		log.trace("Received ", this);
	}
	
	@Override
	public String getMethod() {
		return method.toString();
	}
	
	public HttpMethod getHttpMethod() {
		return method;
	}
	
	@Override
	public Locale getLocale() {
		return locale;
	}
	
	public ServletRequestParameters getRequestParameters() {
		return params;
	}
	
	public String getStringParameter(String name) {
		return params.getStringParameter(name);
	}
	
	public FileParameter getFileParameter(String name) {
		return params.getFileParameter(name);
	}
	
	/**
	 * @return the parameter value as String, null if none
	 */
	@Override
	public String getParameter(String name) {
		return params.getStringParameter(name);
	}
	
	/**
	 * @return the list of string parameter names
	 */
	@Override
	public Enumeration<String> getParameterNames() {
		return Iterators.asEnumeration(params.toParameterMap().keySet().iterator());
	}
	/*
	@Override
	public Map getParameterMap() {
		return params.toParameterMap();
	}*/
	
	/**
	 * Assign the current request to this Thread, to be used anywhere
	 */
	static void setThreadLocal(LoomServletRequest request) {
		threadLocal.set(request);
	}
	
	/**
	 * Clear the request bound to this Thread, to avoid memory leaks
	 */
	public static void clearThreadLocal() {
		threadLocal.remove();
	}
	
	/**
	 * Note that this method does  not come for free. If you can choose, you 
	 * should get the request reference by some other means.
	 * @return the current request bound to this thread.
	 */
	public static LoomServletRequest getThreadLocal() {
		return threadLocal.get();
	}
	
	/**
	 * Change the ThreadLocal implementation
	 * @param threadLocalImpl one of ThreadLocal or InheritedThreadLocal
	 */
	public static void setThreadLocalImpl(ThreadLocal<LoomServletRequest> threadLocalImpl) {
		threadLocal = threadLocalImpl;
	}

	public void addFileParameter(String name, FileParameter value) {
		params.addFileParameter(name, value);
	}
	
	public void addStringParameter(String name, String value) {
		params.addStringParameter(name, value);
	}
	
	public MessagesRepository getMessagesRepository() {
		return messagesRepository;
	}

	public ParsedAction getParsedAction() {
		return parsedAction;
	}

	public void setParsedAction(ParsedAction parsedAction) {
		this.parsedAction = parsedAction;
		this.setAttribute(RequestAttributeNames.PARSED_ACTION, parsedAction);
		this.setAttribute(RequestAttributeNames.ACTION, parsedAction.getAction());
	}

	protected void setMessagesRepository(MessagesRepository messagesRepository) {
		this.messagesRepository = messagesRepository;
		this.locale = messagesRepository.getLocale();
		setAttribute(RequestAttributeNames.MESSAGES_REPOSITORY, messagesRepository.asMap());
	}

	public FormTag getCurrentForm() {
		return currentForm;
	}

	public void setCurrentForm(FormTag currentForm) {
		if (this.currentForm != null && currentForm != null) {
			throw new IllegalArgumentException("Nested forms are not allowed in HTML and are not supported by Loom.");
		}
		this.currentForm = currentForm;
	}

	public String generateId(String prefix) {
		return prefix + generatedIdCount++;
	}
	
	public String generateIndex(String name) {
		String indexAttributeName = RequestAttributeNames.FIELD_INDEX_PREFIX + name;
		Integer current = (Integer) getAttribute(indexAttributeName);
		if (current == null) {
			current = 0;
		}
		String result = name + "[" + current + "]";
		setAttribute(indexAttributeName, current + 1);
		return result;
	}

	public Cookie getCookie(String cookieName) {
		return CookieUtils.getCookie(cookieName, getCookies());
	}

	public String getRequestUrlWithParameters() {
		return toUrlBuilder().getURL();
	}
	
	public UrlBuilder toUrlBuilder() {
		UrlBuilder url = createUrl();
		if (parsedAction != null) {
			url.setEvent(parsedAction.getEvent());
		} else {
			url.setRawUrl(getRequestURI());
		}
		url.setIncludeRequestParameters(true);
		return url;
		
	}

	public boolean isAjax() {
		// jQuery and prototype will use "XMLHttpRequest", but ext will use "Ext.basex"
		return getHeader(RequestHeaderNames.X_REQUESTED_WITH) != null;
	}

	public void setLocale(Locale locale) {
		this.locale = locale;
	}
	
	public boolean getAttributeAsBoolean(String attributeName) {
		Boolean b = (Boolean) getAttribute(attributeName);
		return b != null && b.booleanValue();
	}
	
	public void setScopedAttribute(String name, Object value, Scope scope) {
		switch (scope) {
			case PAGE:
				throw new IllegalArgumentException("Cannot set PAGE scoped attribute, use JSP helper classes instead (PageContext, AbstractLoomTag)");
			case REQUEST:
				setAttribute(name, value);
				break;
			case FLASH:
				getNextFlashContext(true).setAttribute(name, value);
				break;
			case SESSION:
				getSession(true).setAttribute(name, value);
				break;
			case APPLICATION:
				servletContext.setAttribute(name, value);
				break;
			default:
				throw new IllegalArgumentException("Unknown scope: " + scope);
		}
	}
	
	public void removeScopedAttribute(String name, Scope scope) {
		switch (scope) {
		case PAGE:
			throw new IllegalArgumentException("Cannot remove PAGE scoped attribute, use JSP helper classes instead (PageContext, AbstractLoomTag)");
		case REQUEST:
			removeAttribute(name);
			break;
		case FLASH:
			getNextFlashContext(true).removeAttribute(name);
			break;
		case SESSION:
			HttpSession session = getSession(false);
			if (session != null) {
				session.removeAttribute(name);
			}
			break;
		case APPLICATION:
			servletContext.removeAttribute(name);
			break;
		default:
			throw new IllegalArgumentException("Unknown scope: " + scope);
		}
		
	}

	public Object getScopedAttribute(String name, Scope... scopes) {
		if (scopes == null || scopes.length == 0) {
			return getScopedAttribute(name, VALID_SCOPES);
		} 
		for (Scope scope: scopes) {
			Object value;
			switch (scope) {
				case PAGE:
					throw new IllegalArgumentException("Cannot get PAGE scoped attribute, use JSP helper classes instead (PageContext, AbstractLoomTag)");
				case REQUEST:
					value = getAttribute(name);
					break;
				case FLASH:
					FlashContext instance = getPreviousFlashContext();
					value = instance == null? null : instance.getAttribute(name);
					break;
				case SESSION:
					HttpSession session = getSession(false); 
					value = session ==  null? null : session.getAttribute(name);
					break;
				case APPLICATION:
					value = servletContext.getAttribute(name);
					break;
				default:
					throw new IllegalArgumentException("Unknown scope: " + scope);
			}
			if (value !=  null) {
				return value;
			}
		}
		return null;
		
	}
	
	public FlashContext getNextFlashContext(boolean create) {
		FlashContext instance = (FlashContext) getAttribute(FLASH_NEXT);
		if (instance == null && create) {
			instance = flashContextFactory.create(this);
			setAttribute(FLASH_NEXT, instance);
		}
		return instance;
	}

	public FlashContext getPreviousFlashContext() {
		FlashContext instance = (FlashContext) getAttribute(FLASH_PREV);
		if (instance == null) {
			String flashID = getParameter(RequestParameterNames.FLASH_ID);
			if (flashID != null) {
				instance = flashContextFactory.retrieve(this, flashID);
				if (instance != null) {
					setAttribute(FLASH_PREV, instance);
				}
			}
		}
		return instance;
	}

	/**
	 * Get the list of http headers as a Map<String, String[]>.  
	 */
	@SuppressWarnings("unchecked")
	private ListMultimap<String, String> getHeadersMap() {
		ListMultimap<String, String> values = ArrayListMultimap.create();
		for (Enumeration<String> e = getHeaderNames(); e.hasMoreElements(); ) {
			String key = e.nextElement();
			for (Enumeration<String> e2 = getHeaders(key); e2.hasMoreElements(); ) {
				values.put(key, e2.nextElement());
			}
		}
		return values;
	}
	
	
	/**
	 * Do NOT rely on this class format to do anything important, since this format is very likely to change in the future
	 * @return a String representation of this request: attributes, http headers, etc
	 */
	@Override
	public String toString() {
		ToStringBuilder builder = new ToStringBuilder(this, ToStringStyle.MULTI_LINE_STYLE);
		builder.append("uri", getRequestURI());
		builder.append("parsedAction", parsedAction == null? null : parsedAction.getEvent().getPrintName());
		builder.append("locale", locale);
		builder.append("browser", browserData.getUserAgent());
		builder.append("headers", getHeadersMap());
		builder.append("params", params.toString());
		StringBuilder attrs = new StringBuilder(100).append("{");
		for (Enumeration<?> e = getAttributeNames(); e.hasMoreElements(); ) {
			String key = (String) e.nextElement();
			Object value = getAttribute(key);
			if (value instanceof HttpServletRequest) { // to avoid endless looping
				value = value.getClass().getName() + "@" + Integer.toHexString(value.hashCode());
			}
			attrs.append(key).append("=").append(value == null? "<null>" : value);
			if (e.hasMoreElements()) {
				attrs.append(",");
			}
		}
		attrs.append("}");
		builder.append("attributes", attrs);
		return builder.toString();
	}
	
	public UrlBuilder createUrl() {
		return new UrlBuilder(this);
	}
	
	public UrlBuilder createUrl(Class<? extends Action> actionClass, String event) {
		return new UrlBuilder(this, actionClass, event);
	}
	
	public UrlBuilder createUrl(String url) {
		return new UrlBuilder(this, url);
	}
	
	public BrowserData getBrowserData() {
		return browserData;
	}

	public void setBrowserData(BrowserData browserData) {
		this.browserData = browserData;
	}

}
