/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.servlet;

import java.io.IOException;
import java.util.Iterator;

import javax.servlet.FilterChain;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

import org.loom.action.Action;
import org.loom.action.ActionContext;
import org.loom.exception.ExceptionHandler;
import org.loom.exception.WorkflowInterruption;
import org.loom.i18n.Message;
import org.loom.i18n.MessageLevel;
import org.loom.i18n.Messages;
import org.loom.log.Log;
import org.loom.mapping.ActionMapping;
import org.loom.mapping.ActionMappingRepository;
import org.loom.mapping.Event;
import org.loom.mapping.ParsedAction;
import org.loom.resolution.Resolution;
import org.loom.resolution.ResolutionFactory;

/**
 * The execution flow of a request. 
 * @author Ignacio Coloma
 */
public class Workflow {

	/** the servlet context */
	private ServletContext servletContext;
	
	/** the resolutionFactory to apply for normal resolution*/
	private ResolutionFactory resolutionFactory;
	
	private ActionMappingRepository actionMappingRepository;
	
	/** true to redirect if the ending slash is missing, default true */
	private boolean redirectMissingSlash = true;
	
	private ExceptionHandler exceptionHandler;
	
	private static Log log = Log.getLog(Workflow.class);
	
	public Workflow(ServletContext servletContext) {
		this.servletContext = servletContext;
	}

	public void doFilter(LoomServletRequest request, LoomServletResponse response, FilterChain chain) throws IOException, ServletException {
		Event event = actionMappingRepository.getEvent(request.getServletPath(), request);
		
		if (event != null) {
			// if an event was found, invoke
			execute(event, request, response);
		} else if (redirectMissingSlash && "GET".equals(request.getMethod()) && actionMappingRepository.getEvent(request.getServletPath() + "/", request) != null) {
			// if an event was not found because the URL is missing a slash, redirect to the corrected URL 
			String parameters = request.getQueryString();
			response.sendRedirect(request.getRequestURI() + "/" + (parameters == null? "" : "?" + parameters) );
		} else {
			// no action was found. Continue the chain, maybe the request is a file 
			chain.doFilter(request, response);
		}

	}
	
	public void execute(Event event, LoomServletRequest request, LoomServletResponse response) throws ServletException, IOException {
		
		// create action and event 
		ActionMapping mapping = event.getActionMapping();
		Messages messages = new Messages();
		Action a = actionMappingRepository.createActionInstance(mapping);
		ParsedAction action = new ParsedAction(mapping, a, event, messages, request, response);
		((LoomServletRequestImpl)request).setParsedAction(action);
		Resolution resolution = null;

		// if there is any error from a previous request (a redirect), restore them
		messages.restoreFromRedirect(request);
		
		try {
		
			try {
				action.afterCreate();
				ActionContext actionContext = createActionContext(request, response);
				actionContext.setResolutionFactory(resolutionFactory);
				actionContext.setEvent(event);
				a.setContext(actionContext);
				
				// populate action
				action.beforePopulate();
				action.populate(request.getRequestParameters());
				
				// validate
				action.beforeValidate();
				event.validate(action);
				
				// if there are errors originated in this request
				for (Iterator<Message> it = messages.getAllMessages().iterator(); it.hasNext(); ) {
					Message msg = it.next();
					if (msg.getLevel() == MessageLevel.ERROR && !msg.isDeserialized()) {
						// give the action an opportunity to redirect somewhere else
						action.onError();
						
						// default behavior, redirect to the origin form
						exceptionHandler.redirectWithErrors(request, response);
						action.doFinally(null);
						return;
					}
				}
				
				// beforeExecute must be here! See #LOOM-242
				action.beforeExecute();
				resolution = action.invokeEvent();
				
			} catch (WorkflowInterruption interruption) {
				
				resolution = interruption.getResolution();
				log.trace("Workflow has been interrupted on phase ", interruption.getPhase(), " to return ", resolution);
				
			} catch (RuntimeException exception) {
				
				exceptionHandler.handle(request, response, exception);
				action.doFinally(null);
				return;
				
			}

			// execute resolution
			action.beforeResolution(resolution);
			resolution.resolve(request, response);
			action.doFinally(null);
			
			// if there are error messages not included in the HTML output, log them
			// (except if this is a redirect or error)
			if (response.getStatus() == HttpServletResponse.SC_OK) {
				messages.logUnrenderedMessages(request.getMessagesRepository());
			}
			
		} catch (IOException e) {
			doFinally(action, e);
		} catch (ServletException e) {
			doFinally(action, e);
		} catch (RuntimeException e) {
			doFinally(action, e);
		}

	}

	/**
	 * Executes the doFinally block. If any exception is thrown, logs it but throws the original exception.
	 */
	private <T extends Exception> void doFinally(ParsedAction action, T exception) throws T {
		try {
			action.doFinally(exception);
			if (exception != null) {
				throw exception;
			}
		} catch (RuntimeException e1) {
			if (exception == null) {
				throw e1;
			}
			log.error(e1);
			throw exception;
		}
	}
	
	/**
	 * Assigns a new ActionContext to the action.
	 * Each action class can specify its own ActionContext class, which will be detected and instantiated at runtime
	 */
	protected ActionContext createActionContext(LoomServletRequest request, LoomServletResponse response) {
		ActionContext context = new ActionContext();
		context.setRequest(request);
		context.setResponse(response);
		context.setServletContext(servletContext);
		return context;
	}	
	
	public ResolutionFactory getResolutionFactory() {
		return resolutionFactory;
	}

	public void setResolutionFactory(ResolutionFactory resolutionFactory) {
		this.resolutionFactory = resolutionFactory;
	}

	public void setActionMappingRepository(
			ActionMappingRepository actionMappingRepository) {
		this.actionMappingRepository = actionMappingRepository;
	}

	public void setExceptionHandler(ExceptionHandler exceptionTranslator) {
		this.exceptionHandler = exceptionTranslator;
	}



}
