package org.loom.spring;

import java.util.Arrays;
import java.util.List;
import java.util.Set;

import org.loom.action.Action;
import org.loom.annotation.processor.AnnotationProcessor;
import org.loom.annotation.processor.AnnotationProcessorRepository;
import org.loom.annotation.processor.BasicAnnotationProcessor;
import org.loom.annotation.processor.ColumnAnnotationProcessor;
import org.loom.annotation.processor.CommonAnnotationsProcessor;
import org.loom.annotation.processor.RestAnnotationProcessor;
import org.loom.annotation.processor.SecuredAnnotationProcessor;
import org.loom.binding.PropertyWrapperFactory;
import org.loom.config.ClassNames;
import org.loom.config.Config;
import org.loom.i18n.MessagesRepositoryFactoryImpl;
import org.loom.log.Log;
import org.loom.mapping.ActionMappingRepository;
import org.loom.mapping.ReloadableActionMappingRepository;
import org.loom.mapping.RestActionMappingFactory;
import org.loom.resources.InMemoryWebResourceCompiler;
import org.loom.resources.ToDiskWebResourceCompiler;
import org.loom.resources.WebResourceBundleFactoryImpl;
import org.loom.resources.WebResourceBundleImpl;
import org.loom.resources.WebResourceBundleRepository;
import org.loom.resources.WebResourceType;
import org.loom.servlet.LoomServletRequestFactory;
import org.loom.util.ClassUtils;
import org.loom.util.LocaleUtils;
import org.loom.util.StringUtils;
import org.springframework.beans.factory.ListableBeanFactory;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.support.AbstractBeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.xml.AbstractBeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.context.annotation.ComponentScanBeanDefinitionParser;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

import com.google.common.collect.Sets;

/**
 * Parse the &lt;config> custom tag in spring config files.
 * 
 * @author icoloma
 * 
 */
public class ConfigBeanDefinitionParser extends AbstractBeanDefinitionParser {

	private static String NAMESPACE = "http://loom.extrema-sistemas.org/schema/loom";
	
	private BeanDefinitionRegistry beanRegistry;
	
	private static Log log = Log.getLog(ConfigBeanDefinitionParser.class);
	
	@Override
	protected AbstractBeanDefinition parseInternal(Element element, ParserContext parserContext) {
		BeanDefinitionBuilder factory = BeanDefinitionBuilder.rootBeanDefinition(Config.class);
		beanRegistry = parserContext.getReaderContext().getRegistry();
		
		// add infraestructure classses
		addInfrastructureClasses();
		
		// action-scan (required)
		parseActionScan(element, parserContext);
		
		// annotation-processors
		addConditionalAnnotationProcessors();
		
		// web-resources
		Element resources = DomUtils.getChildElementByTagName(element, "web-resources");
		if (resources != null) {
			factory.addPropertyValue("webResourceBundleRepository", parseWebResourceBundles(resources, parserContext));
		}
		
		// LoomSeervletRequestFactory
		initLoomServletRequestFactory(element);
		factory.addPropertyReference("loomServletRequestFactory", "loomServletRequestFactory");
		
		// actionMappingRepository
		factory.addPropertyValue("actionMappingRepository", createActionMappingRepository());
		
		// config attributes
		String development = element.getAttribute("development");
		if (!StringUtils.isEmpty(development)) {
			factory.addPropertyValue("development", development);
		}
		String defaultSSLPolicy = element.getAttribute("defaultSSLPolicy");
		if (!StringUtils.isEmpty(defaultSSLPolicy)) {
			factory.addPropertyValue("defaultSSLPolicy", defaultSSLPolicy);
		}
		
		// postprocessors
		BeanDefinitionBuilder configPostProcessor = BeanDefinitionBuilder.rootBeanDefinition(ConfigPostProcessor.class);
		beanRegistry.registerBeanDefinition("configPostProcessor", configPostProcessor.getBeanDefinition());
		
		return factory.getBeanDefinition();
	}
	
	protected void initLoomServletRequestFactory(Element element) {
		if (!beanRegistry.containsBeanDefinition("loomServletRequestFactory")) {
			BeanDefinitionBuilder factory = BeanDefinitionBuilder.rootBeanDefinition(LoomServletRequestFactory.class);
			beanRegistry.registerBeanDefinition("loomServletRequestFactory", factory.getBeanDefinition());
		}
		BeanDefinition messagesRepositoryFactory = parseMessages(element);
		if (!beanRegistry.containsBeanDefinition("messagesRepositoryFactory")) {
			beanRegistry.registerBeanDefinition("messagesRepositoryFactory", messagesRepositoryFactory);
		}
	}

	/**
	 * Add some annotation processors if the dependent classes are found in the classpath
	 */
	protected void addConditionalAnnotationProcessors() { 
		// register repository
		BeanDefinitionBuilder factory = BeanDefinitionBuilder.rootBeanDefinition(AnnotationProcessorRepository.class);
		beanRegistry.registerBeanDefinition(AnnotationProcessorRepository.class.getSimpleName(), factory.getBeanDefinition());
		
		if (ClassUtils.isPresent("javax.ws.rs.Path")) {
			log.info("Adding JAX-RS annotations support.");
			addAnnotationProcessor(RestAnnotationProcessor.class);
		}
		if (ClassUtils.isPresent("javax.persistence.Entity")) {
			log.info("Adding jpa annotations support.");
			addAnnotationProcessor(ColumnAnnotationProcessor.class);
			addAnnotationProcessor(BasicAnnotationProcessor.class);
		}
		addSecurityAnnotationProcessors();
	}
	
	protected void addSecurityAnnotationProcessors() { 
		ListableBeanFactory beanFactory = (ListableBeanFactory) beanRegistry;
		if (ClassUtils.isPresent(ClassNames.COMMONS_ANNOTATIONS) && beanFactory.getBeanNamesForType(CommonAnnotationsProcessor.class).length == 0) {
			addAnnotationProcessor(CommonAnnotationsProcessor.class);
		}
		if (ClassUtils.isPresent(ClassNames.SPRING_SECURITY) && beanFactory.getBeanNamesForType(SecuredAnnotationProcessor.class).length == 0) {
			addAnnotationProcessor(SecuredAnnotationProcessor.class);
		}
	}

	protected void addAnnotationProcessor(Class<? extends AnnotationProcessor> processorClass) {
		BeanDefinitionBuilder factory = BeanDefinitionBuilder.rootBeanDefinition(processorClass);
		beanRegistry.registerBeanDefinition(processorClass.getSimpleName(), factory.getBeanDefinition());
	}
	
	protected void addInfrastructureClasses() {
		ListableBeanFactory beanFactory = (ListableBeanFactory) beanRegistry;
		if (beanFactory.getBeanNamesForType(PropertyWrapperFactory.class).length == 0) {
			BeanDefinitionBuilder factory = BeanDefinitionBuilder.rootBeanDefinition(PropertyWrapperFactory.class);
			beanRegistry.registerBeanDefinition(PropertyWrapperFactory.class.getSimpleName(), factory.getBeanDefinition());
		}
	}
	

	/**
	 * Parse the action-scan component. The point is generating the following code:
	 * <pre>
	 * &lt;context:component-scan base-package="org.loom.demo.action, org.loom.action.support" scope-resolver="org.loom.spring.PrototypeScopeResolver">
	 *    &lt;context:include-filter type="assignable" expression="org.loom.action.Action"/>
	 * &lt;/context:component-scan>
	 * </pre>
	 * @param element
	 * @param parserContext
	 */
	protected void parseActionScan(Element element, ParserContext parserContext) {
		Element actionScan = DomUtils.getChildElementByTagName(element, "action-scan");
		actionScan.setAttribute("use-default-filters", "false");
		if (StringUtils.isEmpty(actionScan.getAttribute("scope-resolver"))) {
			actionScan.setAttribute("scope-resolver", PrototypeScopeResolver.class.getName());
		}
		Element includeFilter = element.getOwnerDocument().createElementNS(NAMESPACE, "include-filter");
		includeFilter.setAttribute("type", "assignable");
		includeFilter.setAttribute("expression", Action.class.getName());
		actionScan.appendChild(includeFilter);
		parseComponentScan(actionScan, parserContext);
	}

	protected void parseComponentScan(Element element, ParserContext parserContext) {
		ComponentScanBeanDefinitionParser parser = new ComponentScanBeanDefinitionParser();
		parser.parse(element, parserContext);
	}

	private BeanDefinition createActionMappingRepository() {
		Class<?> repositoryClass;
		if (ClassUtils.isPresent(ClassNames.JAVAREBEL)) { 
			log.info("JavaRebel detected in classpath, class reloading is enabled");
			// use introspection to avoid eager loading of classes that may not be in the classpath
			repositoryClass = ReloadableActionMappingRepository.class;
		} else {
			log.info("JavaRebel is not in the classpath, class reloading is disabled");
			repositoryClass = ActionMappingRepository.class;
		}
		BeanDefinitionBuilder amrFactory = BeanDefinitionBuilder.rootBeanDefinition(repositoryClass);
		BeanDefinitionBuilder amfFactory = BeanDefinitionBuilder.rootBeanDefinition(RestActionMappingFactory.class);
		amrFactory.addPropertyValue("actionMappingFactory", amfFactory.getBeanDefinition());
		return amrFactory.getBeanDefinition();
	}

	protected BeanDefinition parseWebResourceBundles(Element element, ParserContext parserContext) {
		BeanDefinitionBuilder webResourceBundleRepository = BeanDefinitionBuilder.rootBeanDefinition(WebResourceBundleRepository.class);
		BeanDefinitionBuilder webResourceBundleFactory = BeanDefinitionBuilder.rootBeanDefinition(WebResourceBundleFactoryImpl.class);
		String storage = element.getAttribute("storage");
		addWebResourceCompiler("_cssCompiler", WebResourceType.CSS, storage);
		addWebResourceCompiler("_jsCompiler", WebResourceType.JS, storage);
		webResourceBundleFactory.addPropertyReference("jsCompiler", "_jsCompiler");
		webResourceBundleFactory.addPropertyReference("cssCompiler", "_cssCompiler");
		
		List<BeanDefinition> bundles = new ManagedList<BeanDefinition>();
		for (Element resource : DomUtils.getChildElementsByTagName(element, "web-resource")) {
			String name = resource.getAttribute("name");
			
			BeanDefinitionBuilder bundle = BeanDefinitionBuilder.rootBeanDefinition(WebResourceBundleImpl.class);
			bundle.addPropertyValue("name", name);
			Set<Resource> resources = parseResourcesList(resource.getTextContent(), parserContext.getReaderContext().getResourceLoader());
			if (resources.isEmpty()) {
				throw new IllegalArgumentException("ResourceBundle " + name + " is empty");
			}
			bundle.addPropertyValue("resources", resources);
			bundle.addPropertyReference("compiler", WebResourceType.identify(resources.iterator().next()) == WebResourceType.CSS? "_cssCompiler" : "_jsCompiler");
			bundles.add(bundle.getBeanDefinition());
		}
		webResourceBundleRepository.addPropertyValue("bundles", bundles);
		webResourceBundleRepository.addPropertyValue("webResourceBundleFactory", webResourceBundleFactory.getBeanDefinition());
		return webResourceBundleRepository.getBeanDefinition();
	}
	
	private BeanDefinition addWebResourceCompiler(String beanName, WebResourceType type, String storage) {
		BeanDefinitionBuilder webResourceCompiler = BeanDefinitionBuilder.rootBeanDefinition(
				"disk".equals(storage)? ToDiskWebResourceCompiler.class : InMemoryWebResourceCompiler.class
		);
		webResourceCompiler.addPropertyValue("type", type);
		BeanDefinition beanDefinition = webResourceCompiler.getBeanDefinition();
		beanRegistry.registerBeanDefinition(beanName, beanDefinition);
		return beanDefinition;
	}

	private Set<Resource> parseResourcesList(String resourcesAsString, ResourceLoader resourceLoader) {
		String[] a = split(resourcesAsString);
		Set<Resource> resources = Sets.newLinkedHashSet();
		for (String resourcePath : a) {
			resources.add(resourceLoader.getResource(resourcePath));
		}
		return resources;
	}
	
	protected BeanDefinition parseMessages(Element root) {
		BeanDefinitionBuilder mrfFactory = BeanDefinitionBuilder.rootBeanDefinition(MessagesRepositoryFactoryImpl.class);
		Element element = DomUtils.getChildElementByTagName(root, "messages");
		if (element != null) {
			String locations = element.getAttribute("locations");
			if (!StringUtils.isEmpty(locations)) {
				mrfFactory.addPropertyValue("locations", Arrays.asList(split(locations)));
			}
			String browserKeys = element.getAttribute("browserMessages");
			if (!StringUtils.isEmpty(browserKeys)) {
				mrfFactory.addPropertyValue("browserMessages", Sets.newHashSet(split(browserKeys)));
			}
			String defaultLocale = element.getAttribute("defaultLocale");
			if (!StringUtils.isEmpty(defaultLocale)) {
				mrfFactory.addPropertyValue("defaultLocale", LocaleUtils.toLocale(defaultLocale));
			}
		}
		return mrfFactory.getBeanDefinition();
	}
	
	protected String[] split(String text) {
		return StringUtils.split(text, ", \n\t");
	}

	@Override
	protected boolean shouldGenerateIdAsFallback() {
		return true;
	}

}
