/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.servlet.jsp.JspException;

import org.loom.config.ConfigProperties;
import org.loom.mapping.ActionMapping;
import org.loom.mapping.Event;
import org.loom.servlet.HttpMethod;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.servlet.scope.FlashContext;
import org.loom.tags.AbstractHtmlTag;
import org.loom.tags.Form;
import org.loom.url.ParameterContainer;
import org.loom.url.UrlBuilder;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;


/**
 * 
 * <a href="http://www.w3.org/TR/html4/interact/forms.html#edef-FORM">HTML form tag</a>.
 * This tag includes a target action and event that are not only used to generate the 
 * target URL but also dictates the validations that will be applied to the input fields.
 * @author icoloma
 */
@Tag(
	dynamicAttributes=true,
	bodyContent=BodyContent.SCRIPTLESS,
	example="&lt;l:form action=\"Mortgages\" event=\"save\">\n" +                
    "    &lt;l:inputHidden name=\"mortgage.id\"/>\n" +
    "    &lt;l:inputText name=\"mortgage.name\"/>\n" +
    "    &lt;l:inputCheckbox name=\"mortgage.starred\"/>\n" +
    "    &lt;l:inputSelect name=\"mortgage.country\" options=\"${action.countries}\" />\n" +
    "    &lt;l:inputFile name=\"mortgage.photo\"/>\n" +
	"&lt;/l:form>")
public class FormTag extends AbstractHtmlTag implements ParameterContainer, Form {
	
	/** should the form fields render a label by default, default true */
	public static boolean DEFAULT_RENDER_LABEL = ConfigProperties.getInstance().getPropertyAsBoolean("formInputField.renderLabel", true);
	
	/** should the form fields render the error message as an HTML component by default, default false */
	public static boolean DEFAULT_SEPARATE_ERRORS = ConfigProperties.getInstance().getPropertyAsBoolean("formInputField.separateErrors", true);

	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-accept-charset">HTML attribute</a> */
	@Attribute
	private String acceptCharset;
	
	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-accept">HTML attribute</a> */
	@Attribute
	private String accept;
	
	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-enctype">HTML attribute</a> */
	@Attribute
	private String enctype;
	
	/** 
	 * <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-method">HTML attribute</a>. 
	 * The default is the first method assigned to the target event.
	 */
	@Attribute
	private String method;
	
	/** <a href="http://www.w3.org/TR/html4/present/frames.html#adef-target">HTML attribute</a> */
	@Attribute
	private String target;
	
	/** true to render field labels by default, default true */
	@Attribute
	private Boolean renderLabel;
	
	/** true to render field errors by default, default true */
	@Attribute
	private Boolean separateErrors;
	
	/** true to disable the whole form, default false */
	@Attribute
	private boolean disabled;
	
	/** set to true to render any nested field as text */
	@Attribute
	private boolean renderAsText;
	
	/** the target action/event pair */
	private UrlBuilder url;
	
	/** 
	 * The <a href="http://msdn2.microsoft.com/en-us/library/ms533486(VS.85).aspx">non-standard attribute</a> to enable/disable browser autocomplete for any field (Firefox and IE).
	 * This will be the default value for any field inside this form. Default is null.
	 */
	@Attribute
	private Boolean autocomplete;

	/**
	 * List of hidden fields that will be inserted at the end of the form
	 */
	private HashMap<String, Object> hiddenFields = new LinkedHashMap<String, Object>();
	
	@Override
	public void setRequest(LoomServletRequest request) {
		super.setRequest(request);
		url = new UrlBuilder(request);
	}
	
	@Override
	protected void initTag() {
		super.initTag();
		request.setCurrentForm(this);
	}
	
	@Override
	protected void endTag() {
		super.endTag();
		request.setCurrentForm(null);
	}
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		// form contents are buffered because InputFileUpload may modify the encType,
		// and error messages must be rendered AFTER all form input fields are done  
		String formBodyContents = doBufferedBody();
		
		// add the typical hidden fields
		calculateHiddenFields();

		// print the tag
		out.print("<form");
		out.printAttribute("action", url.getURL());
		out.printAttribute("accept-charset", acceptCharset);
		out.printAttribute("accept", accept);
		out.printAttribute("enctype", enctype);
		out.printAttribute("method", method);
		out.printAttribute("target", target);
		if (autocomplete !=  null) {
			out.printAttribute("autocomplete", autocomplete? "on" : "off");
		}
		printParentAttributes();
		out.print(">\n");
		
		// print contents
		out.print(formBodyContents);
		
		// add hidden fields
		for (Map.Entry<String, Object> field : hiddenFields.entrySet()) {
			out.printHiddenField(field.getKey(), field.getValue());
		}
		out.print("</form>\n");
		
	}

	/**
	 * Calculate all the hidden fields to add to this form instance
	 */
	protected void calculateHiddenFields() {
		// calculate the form method and the override method, if any
		String methodOverrides = calculateMethodOverride();

		addHiddenField(RequestParameterNames.METHOD_OVERRIDES, methodOverrides);
		FlashContext flash = request.getNextFlashContext(false);
		if (flash != null) {
			addHiddenField(RequestParameterNames.FLASH_ID, flash.getId());
		}
		
		// add interceptor decorations
		url.getEvent().decorate(this);
	}
	
	/**
	 * Add a hidden field to this component. These fields will get appended at the end
	 * of the form.
	 * @param name the name of the input element
	 * @param value the value, its toString form will be used. If null, the entire element will be omitted.
	 */
	public void addHiddenField(String name, Object value) {
		hiddenFields.put(name, value);
	}
	
	/**
	 * When displaying the form tag, this method calculates the value of the OVERRIDE_METHOD parameter, for use with
	 * DELETE and PUT methods which are not supported by the browser.
	 * If this.method has not yet been initialized, assign a suitable value according to the event.
	 * @return the String to be added to the form as an input hidden, null if none
	 * @throws IllegalArgumentException if a method has been specified that is not supported by the assigned event
	 */
	public String calculateMethodOverride() {
		HttpMethod httpMethod = null;
		Event event = getTargetEvent();
		if (method == null) {
			httpMethod = event.getHttpMethods().iterator().next();
			method = HttpMethod.GET.equals(httpMethod)? "GET" : "POST";
		} else {
			httpMethod = HttpMethod.valueOf(method);
			if (httpMethod != HttpMethod.GET && httpMethod != HttpMethod.POST) {
				method = "POST";
			}
		}
		
		// if the method is not supported, error
		if (!event.supports(httpMethod)) {
			throw new IllegalArgumentException("Event " + event.getPrintName() + " does not support the http method " + method);
		}

		String overrideMethod = httpMethod.toString();
		return overrideMethod.equals(method)? null : overrideMethod;
	}
	
	public void setAccept(String accept) {
		this.accept = accept;
	}

	public void setAcceptCharset(String acceptCharset) {
		this.acceptCharset = acceptCharset;
	}

	public void setEnctype(String encType) {
		this.enctype = encType;
	}

	public void setMethod(String method) {
		this.method = method == null? null : method.toUpperCase();
	}

	public void setTarget(String target) {
		this.target = target;
	}

	/** 
	 * An Action instance or Class to get called when this form gets submitted. 
	 * @param action the (unqualified or qualified) name of the action to use for this form
	 */
	@Attribute
	public void setAction(String action) {
		this.url.setAction(action);
	}
	
	/** 
	 * The event that will be invoked when this form is submitted 
	 */
	@Attribute
	public void setEvent(String event) {
		this.url.setEvent(event);
	}
	

	/**
	 * true if the url is relative (do not render schema, server or port), false to render a full URL. 
	 * Default is true 
	 */
	@Attribute 
	public void setRelative(boolean relative) {
		this.url.setRelative(relative);
	}
	
	/**
	 * The scheme to use. If not specified, the current request scheme will be used.
	 * @param scheme the scheme to use. Examples are: "http", "https"
	 */
	@Attribute
	public void setScheme(String scheme) {
		this.url.setScheme(scheme);
	}
	
	/**
	 * The server name to use. If not specified, the current request.getServerName() value will be used.
	 */
	@Attribute
	public void setHostname(String serverName) {
		this.url.setHostname(serverName);
	}
	
	/**
	 * The port to use. If not specified, the current request port will be used.
	 * @param port the port to use. Examples are: 80, 443
	 */
	@Attribute
	public void setPort(Integer port) {
		this.url.setPort(port);
	}
	
	/**
	 * The hash of the url (the part after '#')
	 */
	@Attribute
	public void setHash(String hash) {
		this.url.setHash(hash);
	}

	/**
	 * The mapping as <code>action.event</code>. This method is equivalent to 
	 * setting <code>action</code> and <code>event</code> separately.
	 */
	@Attribute
	public void setMapping(String mapping) {
		this.url.setMapping(mapping);
	}
	
	/**
	 * @return the ActionMapping this form points to
	 */
	public ActionMapping getTargetAction() {
		return url.getActionMapping();
	}
	
	public Event getTargetEvent() {
		return url.getEvent();
	}


	/**
	 * Sets this form to submit multipart data.
	 * Required if the form is going to submit input type="file" fields
	 * @param multipart boolean to set multipart, false if not
	 */
	public void setMultipart(boolean multipart) {
		if (multipart) {
			this.enctype = "multipart/form-data";
			
			// if no HTTP method has been specified, select the first compatible with POST
			if (method == null) {
				for (HttpMethod m : getTargetEvent().getHttpMethods()) {
					if (m != HttpMethod.GET) {
						method = m.toString();
						break;
					}
				}
			} 
			
			// no compatible method was found, or the user sets GET
			if (method == null || "GET".equals(method)) {
				throw new IllegalArgumentException("Cannot upload files with event " + getTargetEvent().getPrintName() + ": the event must support one of POST, PUT or DELETE");
			}
		} else {
			this.enctype = null;
		}

	}

	/**
	 * @return if the renderLabel property has been set, return that; else,
	 * check the global configuration
	 */
	public boolean doRenderLabel() {
		return renderLabel != null? renderLabel.booleanValue() : DEFAULT_RENDER_LABEL;
	}
	
	/**
	 * @return if the renderError property has been set, return that; else,
	 * check the global configuration
	 */
	public boolean doRenderSeparateErrors() {
		return separateErrors != null? separateErrors.booleanValue() : DEFAULT_SEPARATE_ERRORS;
	}

	public void setSeparateErrors(Boolean renderError) {
		this.separateErrors = renderError;
	}

	public void setRenderLabel(Boolean renderLabel) {
		this.renderLabel = renderLabel;
	}

	public boolean isDisabled() {
		return disabled;
	}

	public void setDisabled(boolean disable) {
		this.disabled = disable;
	}

	public boolean isRenderAsText() {
		return renderAsText;
	}

	public void setRenderAsText(boolean readOnly) {
		this.renderAsText = readOnly;
	}

	public void setAutocomplete(Boolean autocomplete) {
		this.autocomplete = autocomplete;
	}

	public String getMethod() {
		return method;
	}
	public ParameterContainer addParameter(String name, Object value) {
		url.add(name, value);
		return this;
	}
	
	public UrlBuilder getUrl() {
		return url;
	}

}
