/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.config.ConfigProperties;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;
import org.loom.util.HtmlSanitizer;


/**
 * <a href="http://www.w3.org/TR/html4/interact/forms.html#edef-TEXTAREA">HTML textarea tag</a>.
 * Any textarea with an associated cols or rows value will do a best guess for the other 
 * attribute using the associated maximum length, if any
 * @author icoloma
 */
@Tag(
		dynamicAttributes=true,
		bodyContent=BodyContent.EMPTY,
		example="&lt;l:textArea name=\"customer.description\" cols=\"80\"/>"
	)
public class InputTextAreaTag extends AbstractTextFieldTag {
	
	private static String DEFAULT_CSS = ConfigProperties.getInstance().getDefaultCss(InputTextAreaTag.class, null);

	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-cols-TEXTAREA">HTML attribute</a> 
	 * If unspecified, it is calculated by dividing the maxLength by the rows value 
	 * */
	@Attribute
	private Integer cols;
	
	/** 
	 * <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-rows-TEXTAREA">HTML attribute</a>. 
	 * If unspecified, it is calculated by dividing the maxLength by the cols value 
	 */
	@Attribute
	private Integer rows;
	
	public InputTextAreaTag() {
		setCssClass(DEFAULT_CSS);
	}
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		if (cols != null && rows == null && maxlength != null) {
			rows = (int) (Math.ceil((double)maxlength / cols));
		}
		if (rows != null && cols == null && maxlength != null) {
			cols = (int) (Math.ceil((double)maxlength / rows));
		}
		
		out.print("<textarea");
		out.printAttribute("cols", cols);
		out.printAttribute("rows", rows);
		printParentAttributes();
		out.print(">");
		String v = getValue();
		if (isEscapeHTML()) {
			v = HtmlSanitizer.sanitize(v);
		}
		if (v != null) {
			out.print(v);
		}
		out.print("</textarea>");
	}
	
	public void setCols(Integer cols) {
		if (cols != null && cols.intValue() <= 0) {
			throw new IllegalArgumentException("Illegal value: " + cols);
		}
		this.cols = cols;
	}

	public void setRows(Integer rows) {
		this.rows = rows;
	}

	public Integer getCols() {
		return cols;
	}

	public Integer getRows() {
		return rows;
	}
	
}
