/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.apache.commons.lang.StringUtils;
import org.loom.servlet.HttpMethod;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.Scope;
import org.loom.tags.AbstractHtmlTag;
import org.loom.tags.Link;
import org.loom.url.ParameterContainer;
import org.loom.url.UrlBuilder;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;


/**
 * <a href="http://www.w3.org/TR/html4/struct/links.html#edef-A">HTML link</a>.
 * Creates a link to a URL or a Action / event pair.
 * If the tag has a body, renders a HTML link around it. Else, it will use the title
 * attribute as a body. 
 * @author icoloma
 *
 */
@Tag(
		dynamicAttributes=true, 
		bodyContent=BodyContent.SCRIPTLESS,
		example=
			"&lt;l:url action=\"BlogEntries\" event=\"open\" param-foo=\"${foo}\"/>\n" +
			"&lt;l:url mapping=\"BlogEntries.open\" />\n" +
			"&lt;l:url href=\"/foo/bar\">Click here&lt;/l:url>"
	)
public class UrlTag extends AbstractHtmlTag implements Link, ParameterContainer {
	
	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-tabindex">HTML attribute</a> */
	@Attribute
	private Integer tabindex;
	
	/** <a href="http://www.w3.org/TR/html4/struct/links.html#adef-charset">HTML attribute</a> */
	@Attribute
	private String charset;

	/** <a href="http://www.w3.org/TR/html4/struct/links.html#adef-hreflang">HTML attribute</a> */
	@Attribute
	private String hreflang;
	
	/** <a href="http://www.w3.org/TR/html4/struct/links.html#adef-rel">HTML attribute</a> */
	@Attribute
	private String rel;
	
	/** <a href="http://www.w3.org/TR/html4/struct/links.html#adef-rev">HTML attribute</a> */
	@Attribute
	private String rev;
	
	/** <a href="http://www.w3.org/TR/html4/struct/objects.html#adef-shape">HTML attribute</a> */
	@Attribute
	private String shape;
	
	/** <a href="http://www.w3.org/TR/html4/present/frames.html#adef-target">HTML attribute</a> */
	@Attribute
	private String target;

	/** <a href="http://www.w3.org/TR/html4/struct/links.html#adef-type-A">HTML attribute</a> */
	@Attribute
	private String type;
	
	/** the HTTP method to use. */
	private HttpMethod method;
	
	/** the scope for the var variable, default 'page' */
	private Scope scope = Scope.PAGE;
	
	/** If specified, store the url value instead of rendering it */
	@Attribute
	private String var;
	
	/** the url this tag points to */
	private UrlBuilder url;

	/** 
	 * true to encode the query string. defaults to true 
	 * (<a href="http://jira.extrema-sistemas.com/jira/browse/LOOM-188">LOOM-188</a>) 
	 */
	@Attribute
	private boolean encodeParameters = true;
	
	@Override
	public void setRequest(LoomServletRequest request) {
		super.setRequest(request);
		url = new UrlBuilder(request);
	}
	
	@Override
	protected void initTag() {
		super.initTag();
		if (url.hasEvent()) {
			url.getEvent().decorate(this);
		}
		url.setEncodeParameters(encodeParameters);
	}

	@Override
	public void doTagImpl() throws JspException, IOException {

		String bufferedBody = doBufferedBody().trim();
		
		// if no action has been specified, use current
		if (url.getActionMapping() == null && url.getRawUrl() == null && request.getParsedAction() != null) {
			url.setActionMapping(request.getParsedAction().getActionMapping());
		}
		
		// if var != null, just set the value and return
		if (var != null) {
			setScopedAttribute(var, url.getURL(), scope);
			return;
		}
		
		// calculate the title if it has not been specified
		if (StringUtils.isEmpty(bufferedBody)) {
			if (getTitle() == null) {
				setTitle(url.calculateTitle());
				if (getTitle() == null) {
					throw new IllegalArgumentException("No tag body and no title / action specified for l:url");
				}
			}
			bufferedBody = getTranslatedTitle();
		}
		
		// print the tag
		out.print("<a");
		out.printAttribute("href", url.getURL());
		out.printAttribute("charset", charset);
		out.printAttribute("hreflang", hreflang);
		out.printAttribute("method", method);
		out.printAttribute("rel", rel);
		out.printAttribute("rev", rev);
		out.printAttribute("shape", shape);
		out.printAttribute("tabindex", tabindex);
		out.printAttribute("target", target);
		out.printAttribute("type", type);
		printParentAttributes();
		out.print(">");
		out.print(bufferedBody);
		out.print("</a>");
		
	}
	
	/** 
	 * If var is not null, this is the scope where the value will be stored.
	 * Possible values are <code>page</code> (default), <code>request</code>,
	 * <code>session</code> and <code>application</code>.  
	 */
	@Attribute
	public void setScope(String scope) {
		this.scope = Scope.fromString(scope);
	}
	
	/**
	 * The action this link points to. Either this or href must be specified
	 */
	@Attribute
	public void setAction(String action) {
		this.url.setAction(action);
	}

	/**
	 * The event this link points to. If action is specified but event is not, the
	 * default event will be assumed. If the action does not define a default event,
	 * an exception will be thrown.
	 * Either action/event or href must be set.
	 */
	@Attribute
	public void setEvent(String event) {
		this.url.setEvent(event);
	}
	
	/**
	 * true if the url is relative (do not render schema, server or port), false to render a full URL. 
	 * Default is true 
	 */
	@Attribute 
	public void setRelative(boolean relative) {
		this.url.setRelative(relative);
	}
	
	/**
	 * The scheme to use. If not specified, the current request scheme will be used.
	 * @param scheme the scheme to use. Examples are: "http", "https"
	 */
	@Attribute
	public void setScheme(String scheme) {
		this.url.setScheme(scheme);
	}
	
	/**
	 * The server name to use. If not specified, the current request.getServerName() value will be used.
	 */
	@Attribute
	public void setHostname(String serverName) {
		this.url.setHostname(serverName);
	}
	
	/**
	 * The port to use. If not specified, the current request port will be used.
	 * @param port the port to use. Examples are: 80, 443
	 */
	@Attribute
	public void setPort(Integer port) {
		this.url.setPort(port);
	}
	
	/**
	 * The mapping as <code>action.event</code>. This method is equivalent to 
	 * setting <code>action</code> and <code>event</code> separately.
	 */
	@Attribute
	public void setMapping(String mapping) {
		this.url.setMapping(mapping);
	}
	
	/**
	 * The hash of the url (the part after '#')
	 */
	@Attribute
	public void setHash(String hash) {
		this.url.setHash(hash);
	}

	/** 
	 * <a href="http://www.w3.org/TR/html4/struct/links.html#adef-href">HTML attribute</a>
	 * The target url. Either this or the action / event pair must be set.
	 */
	@Attribute
	public void setHref(String href) {
		url.setRawUrl(href);
	}
	
	public void setCharset(String charset) {
		this.charset = charset;
	}

	public void setHreflang(String hreflang) {
		this.hreflang = hreflang;
	}

	public void setRel(String rel) {
		this.rel = rel;
	}

	public void setRev(String rev) {
		this.rev = rev;
	}

	public void setShape(String shape) {
		this.shape = shape;
	}

	public void setTarget(String target) {
		this.target = target;
	}

	public void setType(String type) {
		this.type = type;
	}

	public void setVar(String var) {
		this.var = var;
	}
	
	public UrlTag addParameter(String name, Object value) {
		url.add(name, value);
		return this;
	}

	public UrlBuilder getUrl() {
		return url;
	}

	public void setUrl(UrlBuilder url) {
		this.url = url;
	}

	public HttpMethod getMethod() {
		return method;
	}

	public void setMethod(HttpMethod method) {
		this.method = method;
	}

	public boolean isEncodeParameters() {
		return encodeParameters;
	}

	public void setEncodeParameters(boolean encodeParameters) {
		this.encodeParameters = encodeParameters;
	}

	public void setTabindex(Integer tabindex) {
		this.tabindex = tabindex;
	}
	
	public Integer getTabindex() {
		return tabindex;
	}
}
