/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.menu;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.log.Log;
import org.loom.tags.AbstractDecoratedTag;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * Menus are configured inline in the JSP page. 
 * 
 * <p>
 * Performance can be improved by caching menu contents at the server using l:cache
 * </p>
 * @author icoloma
 *
 */
@Tag(
		dynamicAttributes=true,
		bodyContent=BodyContent.SCRIPTLESS,
		example=
			"&lt;l:menu id=\"menu\">\n" +
			"	&lt;l:menuItem action=\"Mortgages\" event=\"listAll\"/>\n" +
			"	&lt;l:menuItem action=\"Leasing\" event=\"listAll\" aliases=\"create, edit, save\"/>\n" +
			"	&lt;l:menuItem href=\"/support/listAllActions.jsp\" label=\"ListAll\"/>\n" +
			"	&lt;l:menuItem action=\"Mail\" event=\"showFolder\" param-folderName=\"INBOX\"/>\n" +
			"&lt;/l:menu>"
	)
public class MenuTag extends AbstractDecoratedTag implements MenuItemTagCollection {

	/** the child MenuItem instances */
	private AbstractMenuData menu;
	
	/** <a href="http://www.w3.org/TR/html4/struct/global.html#adef-id">HTML attribute</a> */
	@Attribute
	private String id;

	/** css class of this menu */
	@Attribute
	private String css;

	/** separator printed between menu items when using decorator "inline" */
	@Attribute
	private String itemSeparator;
	
	private static Log log = Log.getLog(MenuTag.class);
	
	public MenuTag() {
		decorator = "nested";
	}
	
	/**
	 * Neither this or MenuItemTag perform any rendering, the whole work is done by the Decorator.
	 * These tags only assemble a structure of {@link MenuItemData}.
	 */
	@Override
	public void doTagImpl() throws IOException, JspException {
		if (menu == null) { // for testing
			menu = new MenuData();
			menu.setId(id);
			menu.setCssClass(css);
		}
		doBody();
	}

	/** 
	 * @deprecated since Loom 2.0 this attribute will be ignored (see #LOOM-231). Instead, use l:cached
	 */
	@Attribute
	public void setCache(String cache) {
		log.warn("MenuTag.setCache() call ignored. Please use l:cache instead.");
	}

	public void addChild(MenuItemData item) {
		menu.addChild(item);
	}

	public AbstractMenuData getMenu() {
		return menu;
	}

	public void setId(String id) {
		this.id = id;
	}
	
	/**
	 * <a href="http://www.w3.org/TR/html4/struct/global.html#adef-class">HTML attribute</a>
	 */
	@Attribute(name="class")
	public void setCssClass(String cls) {
		this.css = cls;
	}

	public String getItemSeparator() {
		return itemSeparator;
	}

	public void setItemSeparator(String itemSeparator) {
		this.itemSeparator = itemSeparator;
	}

	protected void setMenu(AbstractMenuData menu) {
		this.menu = menu;
	}

	public String getId() {
		return id;
	}

	public String getCssClass() {
		return css;
	}

}
