package org.loom.tags.paged;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.apache.commons.lang.StringUtils;
import org.loom.paged.PagedListData;
import org.loom.paged.SortOrder;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.tags.AbstractTag;
import org.loom.tags.CssBuilder;
import org.loom.url.ParameterContainer;
import org.loom.url.UrlBuilder;
import org.loom.util.HtmlSanitizer;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * A column inside a paged table.
 * @author icoloma
 *
 */
@Tag(
	dynamicAttributes=true,
	bodyContent=BodyContent.SCRIPTLESS,
	example="see <a href=\"pagedTable.html\">pagedTable</a>"
)
public class ColumnTag extends AbstractTag implements ParameterContainer {

	/** The table column header. The default is the property name. */
	@Attribute
	private String header;
	
	/** The cell <code>title</code> <a href="http://www.w3.org/TR/html4/struct/global.html#adef-title">HTML attribute</a> */ 
	@Attribute
	private String cellTitle;
	
	private CssBuilder headerClass = new CssBuilder();
	
	/** the header CSS style. The default is the same value of the <code>style</code> attribute. */
	@Attribute
	private String headerStyle;
	
	/** <a href="http://www.w3.org/TR/html4/present/styles.html#adef-style">HTML attribute</a> */
	@Attribute
	private String style;
	
	/** True to translate the table column header, default true */
	@Attribute
	private boolean translateHeader = true;
	
	/** True to translate the cell title, default true */
	@Attribute
	private boolean translateCellTitle = true;
	
	/** True to make this column sortable, default true */
	@Attribute
	private Boolean sortable;
	
	/** this builder will generate the link to display on this cell, if any */
	private UrlBuilder url;
	
	/** The HTML css class of this column */
	private String cssClass;
	
	/** The surrounding table */
	private PagedTableTag table;
	
	/** the property to  print. If null, the cell body will be used instead */
	@Attribute
	private String property;
	
	private boolean ifClause = true;
	
	@Override
	public void setRequest(LoomServletRequest request) {
		super.setRequest(request);
		url = new UrlBuilder(request);
	}

	@Override
	public void doTagImpl() throws JspException, IOException {
		if (!ifClause) {
			return;
		}
		if (table == null) {
			table = getAncestorWithClass(PagedTableTag.class);
		}
		initSortableDefault();
		
		if (table.getRowIndex() == -1) {
			renderHeader();
		} else {
			renderCell();
		}
	}

	/**
	 * Initialize the sortable attribute to a suitable default (sortable if this column is mapped to a property)
	 */
	private void initSortableDefault() {
		if (sortable == null) {
			sortable = property != null;
		}
	}

	/**
	 * Render a table header (th)
	 */
	private void renderHeader() throws IOException, JspException {
		table.incColumnCount();
		
		PagedListData<?> data = table.getData();
		if (header == null) {
			header = property;
		}
		
		// start header
		out.print("<th");
		if (sortable) {
			if (property == null) {
				throw new IllegalArgumentException("Columns without a property name cannot be sortable");
			}
			if (property.equals(data.getSortProperty())) {
				headerClass.add(data.getSortOrder().getValue());
			}
		}
		out.printAttribute("class", headerClass.getCss());
		out.printAttribute("style", headerStyle != null? headerStyle : style);
		out.print(">");
		
		// print header contents
		String translatedTitle = header == null? " " : translateHeader? repository.guessString(header) : header;
		translatedTitle = evaluateExpression(translatedTitle).toString();
		if (sortable) {
			UrlBuilder headerUrl = table.getUrl().clone();
			if (headerUrl.isEmpty()) {
				throw new IllegalArgumentException("Cannot render sortable table column: the action that retrieves a paged list has not been specified");
			}
			String sp = table.withPrefix(RequestParameterNames.PAGED_SORT_PROPERTY);
			String pso = table.withPrefix(RequestParameterNames.PAGED_SORT_ORDER);
			headerUrl.replace(sp, property);
			String order = property.equals(data.getSortProperty())? data.getSortOrder().getOpposite().getValue() : SortOrder.ASCENDING.getValue();
			headerUrl.replace(pso, order);
			renderHeaderLink(translatedTitle, headerUrl);
		} else {
			out.print(translatedTitle);
		}
		
		// end header
		out.print("</th>");
	}

	protected void renderHeaderLink(String translatedTitle, UrlBuilder headerUrl)
			throws IOException {
		out.print("<a").printAttribute("href", headerUrl.getURL()).print(">");
		out.print(translatedTitle);
		out.print("</a>");
	}

	/**
	 * Render a table cell (td)
	 */
	private void renderCell() throws IOException, JspException {
		// process any nested parameter
		String body = doBufferedBody();
		
		out.print("<td");
		if (!StringUtils.isEmpty(cssClass)) {
			out.printAttribute("class", cssClass);
		}
		if (!StringUtils.isEmpty(cellTitle)) {
			out.printAttribute("title", translateCellTitle? repository.guessString(cellTitle) : cellTitle);
		}
		out.printAttribute("style", style);
		out.print(">");
		if (!url.isEmpty()) {
			out.print("<a").printAttribute("href", url.getURL()).print(">");
		}
		if (property != null && StringUtils.isEmpty(body)) {
			Object value = evaluateExpression("${" + table.getRowName() + "." + property + "}");
			if (value != null) {
				out.print(HtmlSanitizer.sanitize(value.toString()));
			}
		} else {
			out.print(body);
		}
		if (!url.isEmpty()) {
			out.print("</a>");
		}
		out.print("</td>");
	}
	
	/**
	 * If specified, this table cell will include a link pointing to the specified action/event
	 */
	@Attribute
	public void setAction(String action) {
		url.setAction(action);
	}
	
	/**
	 * If specified, this table cell will include a link pointing to the specified action/event
	 */
	@Attribute
	public void setEvent(String event) {
		url.setEvent(event);
	}
	
	/**
	 * If specified, the table cell contents will be surrounded by this link
	 */
	@Attribute
	public void setHref(String href) {
		url.setRawUrl(href);
	}
	
	/**
	 * The mapping as <code>action.event</code>. This method is equivalent to 
	 * setting <code>action</code> and <code>event</code> separately.
	 */
	@Attribute
	public void setMapping(String mapping) {
		this.url.setMapping(mapping);
	}
	
	/**
	 * The hash of the url (the part after '#')
	 */
	@Attribute
	public void setHash(String hash) {
		this.url.setHash(hash);
	}
	
	public ParameterContainer addParameter(String name, Object value) {
		url.add(name, value);
		return this;
	}

	public void setTable(PagedTableTag table) {
		this.table = table;
	}

	public void setHeader(String title) {
		this.header = title;
	}

	public void setTranslateHeader(boolean translateTitle) {
		this.translateHeader = translateTitle;
	}

	/**
	 * <a href="http://www.w3.org/TR/html4/struct/global.html#adef-class">HTML attribute</a>
	 */
	@Attribute(name="class")
	public void setCssClass(String cssClass) {
		this.cssClass = cssClass;
		if (headerClass.isEmpty()) {
			setHeaderClass(cssClass);
		}
	}

	public void setProperty(String property) {
		this.property = property;
	}

	public boolean isSortable() {
		return sortable;
	}

	public void setSortable(boolean sortable) {
		this.sortable = sortable;
	}

	public void setCellTitle(String cellTitle) {
		this.cellTitle = cellTitle;
	}

	public void setTranslateCellTitle(boolean translateCellTitle) {
		this.translateCellTitle = translateCellTitle;
	}

	/** the header CSS class. The default is the same value of the <code>class</code> attribute. */
	@Attribute
	public void setHeaderClass(String headerClass) {
		this.headerClass.setCss(headerClass);
	}
	
	/** 
	 * The table column header. The default is the property name. 
	 * @deprecated use <code>header</code> instead
	 */
	@Attribute
	public void setTitle(String title) {
		setHeader(title);
	}

	public void setHeaderStyle(String headerStyle) {
		this.headerStyle = headerStyle;
	}

	public void setStyle(String style) {
		this.style = style;
	}

	/**
	 * Set to false to skip tag execution.
	 */
	@Attribute
	public void setIf(boolean ifClause) {
		this.ifClause = ifClause;
	}
	
}
