package org.loom.tags.paged;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.i18n.Message;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * Contains a PagedListData of objects to be rendered as a table.
 * This tag uses a target action and event to sort the columns, by default set 
 * to the current action / event pair.
 * 
 * @author icoloma
 *
 */
@Tag(
		dynamicAttributes=true, 
		bodyContent=BodyContent.SCRIPTLESS,
		example=
		"&lt;l:pagedTable data=\"${action.mortgages}\" id=\"mortgages\">\n" +
		"  &lt;l:column sortable=\"false\">\n" +
		"     &lt;l:inputCheckbox name=\"selectedRows\" class=\"selectRow \n" +
		"		checkbox\" value=\"${row.id}\" renderLabel=\"false\"/>\n" +
		"  &lt;/l:column>\n" +
		"  &lt;l:column property=\"id\" />\n" +
		"  &lt;l:column property=\"name\" action=\"Mortgages\" event=\"edit\">\n" +
		"     &lt;l:param name=\"mortgage.id\" value=\"${row.id}\"/>\n" +
		"  &lt;/l:column>\n" +
		"  &lt;l:column property=\"address\" class=\"address\"/>\n" +
		"&lt;/l:pagedTable>"
	)
public class PagedTableTag extends AbstractPagedContainerTag {

	/** the number of nested columns (calculated) */
	private int columnCount;
	
	/** true to render table headers, default true */
	@Attribute
	private boolean renderHeaders = true;
	
	/** set by any nested FooterTag */
	private String footer;

	/**
	 * Render the thead of the table
	 */
	@Override
	protected void renderHeader() throws IOException, JspException {
		out.print("<table").printAttribute("id", getId()).print(">\n");
		
		if (renderHeaders) {
			// start at -1 to print just headers
			setRowIndex(-1);
			out.print("<thead><tr>\n");
			doBody();
			out.print("\n</tr></thead>\n");
		}
		out.print("<tbody>\n");

	}
	
	@Override
	protected void renderEmptyContainer() throws IOException, JspException {
		renderHeader();
		out.print("<tr").printAttribute("class", "noresults").print(">");
		out.print("<td").printAttribute("colspan", columnCount).print(">");
		out.print(repository.translateMessage(new Message(getMessagePrefix() + "empty")));
		out.print("</td></tr>\n");
		renderFooter();
	}
	
	@Override
	protected void renderFooter() throws IOException, JspException {
		out.print("</tbody>\n");
		if (footer != null) {
			out.start("tfoot").print(footer).end("tfoot");
		}
		out.print("</table>\n");
	}
	
	@Override
	protected void renderRowStart() throws IOException, JspException {
		out.print("<tr").printAttribute("class", getRowClass()).print(">");
	}
	
	@Override
	protected void renderRowEnd() throws IOException, JspException {
		out.print("</tr>\n");
	}
	
	/**
	 * Add +1 to the column count
	 */
	public void incColumnCount() {
		columnCount++;
	}

	public void setRenderHeaders(boolean renderHeaders) {
		this.renderHeaders = renderHeaders;
	}

	public void setFooter(String footer) {
		this.footer = footer;
	}	

}
