package org.loom.url;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.loom.servlet.LoomServletRequest;

/**
 * Constructs a url balancing between several host names.
 * A different host will be used depending on the URL to 
 * overcome the browser limit of two concurrent connections per host.
 * @author Nacho
 *
 */
public class BalancedUrlBuilder extends UrlBuilder {

	/**
	 * the hosts list for round robin polling. They can all point to the same IP.
	 */
	private List<String> balancingHostnames;
	
	/**
	 * The list of parameter names to be used while calculating the host name to use.
	 * By default, all parameters will be used to calculate the hostname
	 */
	private Collection<String> balancingParameterNames;

	@Override
	public String getURL(LoomServletRequest request) {
		if (getHostname() == null) {
			setHostname(calculateHostname());
		}
		return super.getURL(request);
	}
	
	/**
	 * Calculate the host name to use. The host name will be calculated using the hashcode of 
	 * the uri and all the parameter values coresponding to balancingParameterNames
	 */
	protected String calculateHostname() {
		int index = 0; 
		if (balancingParameterNames == null) {
			balancingParameterNames = getParameters().keySet();
		}
		for (String name: balancingParameterNames) {
			String value = getParameter(name);
			if (value != null) {
				index += value.hashCode();
			}
		}
		return balancingHostnames.get(Math.abs(index) % balancingHostnames.size());
	}
	
	public List<String> getBalancingHostnames() {
		return balancingHostnames;
	}

	public void setBalancingHostnames(List<String> hosts) {
		this.balancingHostnames = hosts;
	}
	
	public void setBalancingHostnames(String... hosts) {
		setBalancingHostnames(Arrays.asList(hosts));
	}

	public Collection<String> getBalancingParameterNames() {
		return balancingParameterNames;
	}

	public void setBalancingParameterNames(
			Collection<String> balancingParameterNames) {
		this.balancingParameterNames = balancingParameterNames;
	}

	public void setBalancingParameterNames(String... names) {
		setBalancingParameterNames(Arrays.asList(names));
	}
}
