/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.util;

import java.util.ArrayList;
import java.util.Currency;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;

public class LocaleUtils {

	/** the map between {@link Currency} instances and their corresponding HTML entities */
	private static Map<Currency, String> currencyHtmlEntities;
	
	static {
		currencyHtmlEntities = new HashMap<Currency, String>();
		addHtmlEntity(Currency.getInstance("EUR"), "&euro;");
		addHtmlEntity(Currency.getInstance("USD"), "$");
		addHtmlEntity(Currency.getInstance("GBP"), "&pound;");
		addHtmlEntity(Currency.getInstance("JPY"), "&yen;");
	}
	
	/**
	 * Add a currency HTML entity 
	 * @param currency the currency to use as key
	 * @param symbol the HTML text to print to represent that symbol
	 */
	public static void addHtmlEntity(Currency currency, String symbol) {
		currencyHtmlEntities.put(currency, symbol);
	}
	
	/**
	 * 
	 * @param currency the {@link Currency} that will be printed as HTML
	 * @return the HTML text to represent the provided {@link Currency}, null if not found.
	 * Ex: getCurrencySymbol(Currency.getInstance("EUR")) -> "&amp;euro" 
	 */
	public static String getHtmlEntity(Currency currency) {
		return currencyHtmlEntities.get(currency);
	}
	
	/**
	 * Converts a Locale from the ISO form lang_country_variant
	 * to the corresponding Locale instance
	 */
	public static Locale toLocale(String serializedLocale) {
		String[] parts = StringUtils.split(serializedLocale, "_");
		if (parts.length == 0) {
			throw new IllegalArgumentException("Malformed locale string '" + serializedLocale + "'");
		}
		String lang = parts[0];
		String country = parts.length >= 2? parts[1] : "";
		String variant = parts.length >= 3? parts[2] : "";
		return new Locale(lang, country, variant);
	}

	/**
	 * Elaborates a list of localized filenames to check, from more to less specific.
	 * E.g.: fullname="myprops", locale="es_ES", defaultLocale="en_US" will return the following files:
	 * 
	 * myprops_es_ES
	 * myprops_es
	 * myprops_en_US
	 * myprops_en
	 * myprops
	 * @param prefix the prefix of the filename
	 * @param suffix the suffix to apply after the locale, e.g. ".properties"
	 * 
	 */
	public static List<String> getLocalizedFilenames(String prefix, String suffix, Locale... locales) {
		List<String> result = new ArrayList<String>();
		Set<String> visited = new HashSet<String>();
		for (Locale locale : locales) {
			addLocalized(prefix, suffix, locale, result, visited);
		}
		result.add(prefix + suffix);
		return result;
	}
	
	/**
	 * Add all localized versions of a filename to the specified filenames list
	 */
	private final static void addLocalized(String prefix, String suffix, Locale locale, List<String> filenames, Set<String> visited) {
		String language = locale.getLanguage();
		String country = locale.getCountry();
		String variant = locale.getVariant();
		if (variant.length() > 0) {
			addFileName(filenames, visited, prefix + "_" + locale.toString() + suffix);
		}
		if (country.length() > 0) {
			addFileName(filenames, visited, prefix + "_" + language + "_" + country + suffix);
		}
		if (language.length() > 0) {
			addFileName(filenames, visited, prefix + "_" + language + suffix);
		}
	}
	
	private final static void addFileName(List<String> filenames, Set<String> visited, String filename) {
		// a Set could not be used here 
		if (!visited.contains(filename)) {
			filenames.add(filename);
			visited.add(filename);
		}
	}

}
